@@ -5,6 +5,7 @@
  *  Copyright  2010  Mariana Penna Lima
  *  <pennalima@gmail.com>
  ****************************************************************************/
+
 /*
  * numcosmo
  * Copyright (C) Mariana Penna Lima 2012 <pennalima@gmail.com>
@@ -28,15 +29,15 @@
  * @title: NcMultiplicityFunc
  * @short_description: Dark matter halo multiplicity function.
  *
- * The  multiplicity function comprises information about the non-linear regime 
+ * The  multiplicity function comprises information about the non-linear regime
  * of halo (structure) formation. The mass function can be written as
  * \begin{equation}\label{def:multip}
  * \frac{dn (M,z)}{d\ln M} = - \frac{\rho_m (z)}{M} f(\sigma_R, z) \frac{1}{\sigma_R} \frac{d\sigma_R}{d\ln M},
  * \end{equation}
- * where $\rho_m(z)$ is the mean matter density at redshift $z$, $f(\sigma_R, z)$ is the multiplicity function, 
+ * where $\rho_m(z)$ is the mean matter density at redshift $z$, $f(\sigma_R, z)$ is the multiplicity function,
  * and $\sigma_R$ is the variance of the linear density contrast filtered on the length scale $R$ associated to the
- * mass $M$. 
- * 
+ * mass $M$.
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -58,6 +59,7 @@ enum
 {
   PROP_0,
   PROP_MDEF,
+  PROP_DELTA,
   PROP_SIZE,
 };
 
@@ -67,14 +69,15 @@ static void
 nc_multiplicity_func_init (NcMultiplicityFunc *mulf)
 {
   NcMultiplicityFuncPrivate * const self = mulf->priv = nc_multiplicity_func_get_instance_private (mulf);
-  
+
   self->place_holder = 0;
 }
 
 static void
 _nc_multiplicity_func_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
-  NcMultiplicityFunc *mulf               = NC_MULTIPLICITY_FUNC (object);
+  NcMultiplicityFunc *mulf = NC_MULTIPLICITY_FUNC (object);
+
   /* NcMultiplicityFuncPrivate * const self = mulf->priv; */
 
   g_return_if_fail (NC_IS_MULTIPLICITY_FUNC (object));
@@ -84,6 +87,9 @@ _nc_multiplicity_func_set_property (GObject *object, guint prop_id, const GValue
     case PROP_MDEF:
       NC_MULTIPLICITY_FUNC_GET_CLASS (mulf)->set_mdef (mulf, g_value_get_enum (value));
       break;
+    case PROP_DELTA:
+      NC_MULTIPLICITY_FUNC_GET_CLASS (mulf)->set_Delta (mulf, g_value_get_double (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -93,7 +99,8 @@ _nc_multiplicity_func_set_property (GObject *object, guint prop_id, const GValue
 static void
 _nc_multiplicity_func_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
-  NcMultiplicityFunc *mulf               = NC_MULTIPLICITY_FUNC (object);
+  NcMultiplicityFunc *mulf = NC_MULTIPLICITY_FUNC (object);
+
   /* NcMultiplicityFuncPrivate * const self = mulf->priv; */
 
   g_return_if_fail (NC_IS_MULTIPLICITY_FUNC (object));
@@ -103,6 +110,9 @@ _nc_multiplicity_func_get_property (GObject *object, guint prop_id, GValue *valu
     case PROP_MDEF:
       g_value_set_enum (value, nc_multiplicity_func_get_mdef (mulf));
       break;
+    case PROP_DELTA:
+      g_value_set_double (value, nc_multiplicity_func_get_Delta (mulf));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -112,22 +122,68 @@ _nc_multiplicity_func_get_property (GObject *object, guint prop_id, GValue *valu
 static void
 _nc_multiplicity_func_finalize (GObject *object)
 {
-  
   /* Chain up : end */
   G_OBJECT_CLASS (nc_multiplicity_func_parent_class)->finalize (object);
 }
 
-static void _nc_multiplicity_func_set_mdef (NcMultiplicityFunc *mulf, NcMultiplicityFuncMassDef mdef) { g_error ("method set_mdef not implemented by %s.", G_OBJECT_TYPE_NAME (mulf)); }
-static NcMultiplicityFuncMassDef _nc_multiplicity_func_get_mdef (NcMultiplicityFunc *mulf) { g_error ("method get_mdef not implemented by %s.", G_OBJECT_TYPE_NAME (mulf)); return -1; }
-static gdouble _nc_multiplicity_func_eval (NcMultiplicityFunc *mulf, NcHICosmo *cosmo, gdouble sigma, gdouble z) { g_error ("method eval not implemented by %s.", G_OBJECT_TYPE_NAME (mulf)); return 0.0; }
-static gboolean _nc_multiplicity_func_has_correction_factor (NcMultiplicityFunc *mulf);
-static gdouble _nc_multiplicity_func_correction_factor (NcMultiplicityFunc *mulf, NcHICosmo *cosmo, gdouble sigma, gdouble z, gdouble lnM) { g_error ("method correction factor not implemented by %s.", G_OBJECT_TYPE_NAME (mulf)); return 0.0; } 
+static void
+_nc_multiplicity_func_set_mdef (NcMultiplicityFunc *mulf, NcMultiplicityFuncMassDef mdef)
+{
+  g_error ("method set_mdef not implemented by %s.", G_OBJECT_TYPE_NAME (mulf));
+}
 
+static void
+_nc_multiplicity_func_set_Delta (NcMultiplicityFunc *mulf, gdouble Delta)
+{
+  g_error ("method set_Delta not implemented by %s.", G_OBJECT_TYPE_NAME (mulf));
+}
+
+static NcMultiplicityFuncMassDef
+_nc_multiplicity_func_get_mdef (NcMultiplicityFunc *mulf)
+{
+  g_error ("method get_mdef not implemented by %s.", G_OBJECT_TYPE_NAME (mulf));
+
+  return -1;
+}
+
+static gdouble
+_nc_multiplicity_func_get_Delta (NcMultiplicityFunc *mulf)
+{
+  g_error ("method get_Delta not implemented by %s.", G_OBJECT_TYPE_NAME (mulf));
+
+  return -1;
+}
+
+static gdouble
+_nc_multiplicity_func_get_matter_Delta (NcMultiplicityFunc *mulf, NcHICosmo *cosmo, gdouble z)
+{
+  g_error ("method get_matter_Delta not implemented by %s.", G_OBJECT_TYPE_NAME (mulf));
+
+  return -1;
+}
+
+static gdouble
+_nc_multiplicity_func_eval (NcMultiplicityFunc *mulf, NcHICosmo *cosmo, gdouble sigma, gdouble z)
+{
+  g_error ("method eval not implemented by %s.", G_OBJECT_TYPE_NAME (mulf));
+
+  return 0.0;
+}
+
+static gboolean _nc_multiplicity_func_has_correction_factor (NcMultiplicityFunc *mulf);
+
+static gdouble
+_nc_multiplicity_func_correction_factor (NcMultiplicityFunc *mulf, NcHICosmo *cosmo, gdouble sigma, gdouble z, gdouble lnM)
+{
+  g_error ("method correction factor not implemented by %s.", G_OBJECT_TYPE_NAME (mulf));
+
+  return 0.0;
+}
 
 static void
 nc_multiplicity_func_class_init (NcMultiplicityFuncClass *klass)
 {
-  GObjectClass *object_class  = G_OBJECT_CLASS (klass);
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
   object_class->set_property = &_nc_multiplicity_func_set_property;
   object_class->get_property = &_nc_multiplicity_func_get_property;
@@ -136,10 +192,10 @@ nc_multiplicity_func_class_init (NcMultiplicityFuncClass *klass)
   /**
    * NcMultiplicityFunc:mass-def:
    *
-   * It refers to the halo finder used to obtain the multiplicity function (e.g., SO and FoF), and 
+   * It refers to the halo finder used to obtain the multiplicity function (e.g., SO and FoF), and
    * the background density $\rho_\mathrm{bg}$ used in the mass definition \eqref{eq:mrr}.
    * See the enumerator #NcMultiplicityFuncMassDef for more details about the
-   * background density definition. 
+   * background density definition.
    *
    */
   g_object_class_install_property (object_class,
@@ -149,15 +205,35 @@ nc_multiplicity_func_class_init (NcMultiplicityFuncClass *klass)
                                                       "Mass definition",
                                                       NC_TYPE_MULTIPLICITY_FUNC_MASS_DEF, NC_MULTIPLICITY_FUNC_MASS_DEF_MEAN,
                                                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
-  
+
+
+/**
+ * NcMultiplicityFunc:mass-Delta:
+ *
+ *
+ *
+ */
+  g_object_class_install_property (object_class,
+                                   PROP_DELTA,
+                                   g_param_spec_double ("Delta",
+                                                        NULL,
+                                                        "Delta",
+                                                        1.0, G_MAXDOUBLE, 200.0,
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+
+
+
   klass->set_mdef              = &_nc_multiplicity_func_set_mdef;
+  klass->set_Delta             = &_nc_multiplicity_func_set_Delta;
   klass->get_mdef              = &_nc_multiplicity_func_get_mdef;
-  klass->eval                  = &_nc_multiplicity_func_eval; 
+  klass->get_Delta             = &_nc_multiplicity_func_get_Delta;
+  klass->get_matter_Delta      = &_nc_multiplicity_func_get_matter_Delta;
+  klass->eval                  = &_nc_multiplicity_func_eval;
   klass->has_correction_factor = &_nc_multiplicity_func_has_correction_factor;
   klass->correction_factor     = &_nc_multiplicity_func_correction_factor;
 }
 
-static gboolean 
+static gboolean
 _nc_multiplicity_func_has_correction_factor (NcMultiplicityFunc *mulf)
 {
   return FALSE;
@@ -170,7 +246,7 @@ _nc_multiplicity_func_has_correction_factor (NcMultiplicityFunc *mulf)
  * Atomically decrements the reference count of @mulf by one. If the reference count drops to 0,
  * all memory allocated by @mulf is released.
  *
-*/
+ */
 void
 nc_multiplicity_func_free (NcMultiplicityFunc *mulf)
 {
@@ -184,7 +260,7 @@ nc_multiplicity_func_free (NcMultiplicityFunc *mulf)
  * Atomically decrements the reference count of @mulf by one. If the reference count drops to 0,
  * all memory allocated by @mulf is released. Set pointer to NULL.
  *
-*/
+ */
 void
 nc_multiplicity_func_clear (NcMultiplicityFunc **mulf)
 {
@@ -205,6 +281,39 @@ nc_multiplicity_func_set_mdef (NcMultiplicityFunc *mulf, NcMultiplicityFuncMassD
   NC_MULTIPLICITY_FUNC_GET_CLASS (mulf)->set_mdef (mulf, mdef);
 }
 
+/**
+ * nc_multiplicity_func_set_Delta: (virtual set_Delta)
+ * @mulf: a #NcMultiplicityFunc
+ * @Delta: the $Delta$ value
+ *
+ * Sets the Delta for mean or matter mass definition to mdef.
+ *
+ */
+void
+nc_multiplicity_func_set_Delta (NcMultiplicityFunc *mulf, gdouble Delta)
+{
+  NcMultiplicityFuncMassDef mdef = nc_multiplicity_func_get_mdef (mulf);
+
+  switch (mdef)
+  {
+    case NC_MULTIPLICITY_FUNC_MASS_DEF_MEAN:
+      NC_MULTIPLICITY_FUNC_GET_CLASS (mulf)->set_Delta (mulf, Delta);
+      break;
+    case NC_MULTIPLICITY_FUNC_MASS_DEF_CRITICAL:
+      NC_MULTIPLICITY_FUNC_GET_CLASS (mulf)->set_Delta (mulf, Delta);
+      break;
+    case NC_MULTIPLICITY_FUNC_MASS_DEF_VIRIAL:
+      g_error ("NcMultiplicityFuncMassDefVirial does not support Delta def");
+      break;
+    case NC_MULTIPLICITY_FUNC_MASS_DEF_FOF:
+      g_error ("NcMultiplicityFuncMassDefFof does not support Delta def");
+      break;
+    default:
+      g_assert_not_reached ();
+      break;
+  }
+}
+
 /**
  * nc_multiplicity_func_get_mdef: (virtual get_mdef)
  * @mulf: a #NcMultiplicityFunc
@@ -219,6 +328,66 @@ nc_multiplicity_func_get_mdef (NcMultiplicityFunc *mulf)
   return NC_MULTIPLICITY_FUNC_GET_CLASS (mulf)->get_mdef (mulf);
 }
 
+/**
+ * nc_multiplicity_func_get_Delta: (virtual get_Delta)
+ * @mulf: a #NcMultiplicityFunc
+ *
+ * Gets the Delta Value.
+ *
+ * Returns: Delta.
+ */
+gdouble
+nc_multiplicity_func_get_Delta (NcMultiplicityFunc *mulf)
+{
+  return NC_MULTIPLICITY_FUNC_GET_CLASS (mulf)->get_Delta (mulf);
+}
+
+/**
+ * nc_multiplicity_func_get_matter_Delta: (virtual get_matter_Delta)
+ * @mulf: a #NcMultiplicityFunc
+ * @cosmo: a #NcHICosmo
+ * @z: the redshift $z$
+ *
+ * Gets the Delta Value with the matter density definition.
+ *
+ * Returns: Delta.
+ */
+
+gdouble
+nc_multiplicity_func_get_matter_Delta (NcMultiplicityFunc *mulf, NcHICosmo *cosmo, gdouble z)
+{
+  NcMultiplicityFuncMassDef mdef = nc_multiplicity_func_get_mdef (mulf);
+
+  switch (mdef)
+  {
+    case NC_MULTIPLICITY_FUNC_MASS_DEF_MEAN:
+
+      return nc_multiplicity_func_get_Delta (mulf);
+
+      break;
+    case NC_MULTIPLICITY_FUNC_MASS_DEF_CRITICAL:
+    {
+      const gdouble E2      = nc_hicosmo_E2 (cosmo, z);
+      const gdouble Omega_m = nc_hicosmo_E2Omega_m (cosmo, z) / E2;
+      const gdouble Delta_z = nc_multiplicity_func_get_Delta (mulf) / Omega_m;
+
+      return Delta_z;
+
+      break;
+    }
+    case NC_MULTIPLICITY_FUNC_MASS_DEF_VIRIAL:
+      g_error ("NcMultiplicityFuncMassDefVirial does not support Delta def");
+      break;
+    case NC_MULTIPLICITY_FUNC_MASS_DEF_FOF:
+      g_error ("NcMultiplicityFuncMassDefFof does not support Delta def");
+      break;
+    default:
+      g_assert_not_reached ();
+      break;
+  }
+  return 0.0;
+}
+
 /**
  * nc_multiplicity_func_eval: (virtual eval)
  * @mulf: a #NcMultiplicityFunc
@@ -229,7 +398,7 @@ nc_multiplicity_func_get_mdef (NcMultiplicityFunc *mulf)
  * FIXME
  *
  * Returns: FIXME
-*/
+ */
 gdouble
 nc_multiplicity_func_eval (NcMultiplicityFunc *mulf, NcHICosmo *cosmo, gdouble sigma, gdouble z)
 {
@@ -240,11 +409,11 @@ nc_multiplicity_func_eval (NcMultiplicityFunc *mulf, NcHICosmo *cosmo, gdouble s
  * nc_multiplicity_func_has_correction_factor: (virtual has_correction_factor)
  * @mulf: a #NcMultiplicityFunc
  *
- * Checks if the multiplicity function has a correction factor, e.g., NcMultiplicityFuncBocquet (when using 
- * critical density).
+ * Checks if the multiplicity function has a correction factor, e.g., NcMultiplicityFuncBocquet (when using
+ * matter density).
  *
  * Returns: a gboolean
-*/
+ */
 gboolean
 nc_multiplicity_func_has_correction_factor (NcMultiplicityFunc *mulf)
 {
@@ -262,9 +431,10 @@ nc_multiplicity_func_has_correction_factor (NcMultiplicityFunc *mulf)
  * FIXME
  *
  * Returns: FIXME
-*/
+ */
 gdouble
 nc_multiplicity_func_correction_factor (NcMultiplicityFunc *mulf, NcHICosmo *cosmo, gdouble sigma, gdouble z, gdouble lnM)
 {
   return NC_MULTIPLICITY_FUNC_GET_CLASS (mulf)->correction_factor (mulf, cosmo, sigma, z, lnM);
-}
\ No newline at end of file
+}
+
