@@ -73,7 +73,7 @@
 #include "lss/nc_wl_surface_mass_density.h"
 #include "nc_distance.h"
 #include "lss/nc_halo_density_profile.h"
-#include "math/integral.h"
+#include "math/ncm_integrate.h"
 #include "math/ncm_c.h"
 #include "math/ncm_cfg.h"
 #include "math/ncm_spline_cubic_notaknot.h"
