@@ -14,19 +14,19 @@
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
- * 
+ *
  * numcosmo is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License along
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
  * SECTION:ncm_data_gauss_cov_mvnd
- * @title: NcmDataGaussCov
+ * @title: NcmDataGaussCovMVND
  * @short_description: Multivariate Normal Distribution -- covariance provided.
  *
  * Multivariate Normal distribution which uses the covariance matrix as input.
@@ -45,23 +45,21 @@
 #ifndef NUMCOSMO_GIR_SCAN
 #endif /* NUMCOSMO_GIR_SCAN */
 
-struct _NcmDataGaussCovMVNDPrivate
+struct _NcmDataGaussCovMVND
 {
-  gint unused;
+  NcmDataGaussCov parent_instance;
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE (NcmDataGaussCovMVND, ncm_data_gauss_cov_mvnd, NCM_TYPE_DATA_GAUSS_COV);
+G_DEFINE_TYPE (NcmDataGaussCovMVND, ncm_data_gauss_cov_mvnd, NCM_TYPE_DATA_GAUSS_COV);
 
 static void
 ncm_data_gauss_cov_mvnd_init (NcmDataGaussCovMVND *gauss_mvnd)
 {
-  gauss_mvnd->priv = ncm_data_gauss_cov_mvnd_get_instance_private (gauss_mvnd);
 }
 
 static void
 ncm_data_gauss_cov_mvnd_finalize (GObject *object)
 {
-
   /* Chain up : end */
   G_OBJECT_CLASS (ncm_data_gauss_cov_mvnd_parent_class)->finalize (object);
 }
@@ -71,8 +69,8 @@ static void _ncm_data_gauss_cov_mvnd_mean_func (NcmDataGaussCov *gauss, NcmMSet
 static void
 ncm_data_gauss_cov_mvnd_class_init (NcmDataGaussCovMVNDClass *klass)
 {
-  GObjectClass* object_class        = G_OBJECT_CLASS (klass);
-  NcmDataGaussCovClass* gauss_class = NCM_DATA_GAUSS_COV_CLASS (klass);
+  GObjectClass *object_class        = G_OBJECT_CLASS (klass);
+  NcmDataGaussCovClass *gauss_class = NCM_DATA_GAUSS_COV_CLASS (klass);
 
   object_class->finalize = ncm_data_gauss_cov_mvnd_finalize;
 
@@ -81,22 +79,22 @@ ncm_data_gauss_cov_mvnd_class_init (NcmDataGaussCovMVNDClass *klass)
   gauss_class->cov_func  = NULL;
 }
 
-static void 
+static void
 _ncm_data_gauss_cov_mvnd_mean_func (NcmDataGaussCov *gauss, NcmMSet *mset, NcmVector *vp)
 {
   NcmModelMVND *model_mvnd = NCM_MODEL_MVND (ncm_mset_peek (mset, ncm_model_mvnd_id ()));
-  
+
   ncm_model_mvnd_mean (model_mvnd, vp);
 }
 
 /**
  * ncm_data_gauss_cov_mvnd_new:
  * @dim: dimension of the MVND
- * 
+ *
  * Creates a new @dim-dimensional MVND.
- * 
+ *
  * Returns: the newly created object.
- */ 
+ */
 NcmDataGaussCovMVND *
 ncm_data_gauss_cov_mvnd_new (const guint dim)
 {
@@ -104,6 +102,7 @@ ncm_data_gauss_cov_mvnd_new (const guint dim)
                                                   "n-points", dim,
                                                   "use-norma", TRUE,
                                                   NULL);
+
   return gauss_mvnd;
 }
 
@@ -116,16 +115,17 @@ ncm_data_gauss_cov_mvnd_new (const guint dim)
  * @mean_min: minimum mean $\mu_i$
  * @mean_max: maximum mean $\mu_i$
  * @rng: a #NcmRNG
- * 
- * Creates a new @dim-dimensional MVND and generate using @rng a mean 
+ *
+ * Creates a new @dim-dimensional MVND and generate using @rng a mean
  * and correlation matrix using the parameters above.
- * 
+ *
  * Returns: the newly created object.
- */ 
+ */
 NcmDataGaussCovMVND *
 ncm_data_gauss_cov_mvnd_new_full (const guint dim, const gdouble sigma_min, const gdouble sigma_max, const gdouble cor_level, const gdouble mean_min, const gdouble mean_max, NcmRNG *rng)
 {
   NcmDataGaussCovMVND *gauss_mvnd = ncm_data_gauss_cov_mvnd_new (dim);
+
   ncm_data_gauss_cov_mvnd_gen_cov_mean (gauss_mvnd, sigma_min, sigma_max, cor_level, mean_min, mean_max, rng);
 
   return gauss_mvnd;
@@ -134,9 +134,9 @@ ncm_data_gauss_cov_mvnd_new_full (const guint dim, const gdouble sigma_min, cons
 /**
  * ncm_data_gauss_cov_mvnd_ref:
  * @data_mvnd: a #NcmDataGaussCovMVND
- * 
+ *
  * Increases the reference count of @data_mvnd by one.
- * 
+ *
  * Returns: (transfer full): @data_mvnd
  */
 NcmDataGaussCovMVND *
@@ -148,11 +148,11 @@ ncm_data_gauss_cov_mvnd_ref (NcmDataGaussCovMVND *data_mvnd)
 /**
  * ncm_data_gauss_cov_mvnd_free:
  * @data_mvnd: a #NcmDataGaussCovMVND
- * 
+ *
  * Decreases the reference count of @data_mvnd by one.
- * 
+ *
  */
-void 
+void
 ncm_data_gauss_cov_mvnd_free (NcmDataGaussCovMVND *data_mvnd)
 {
   g_object_unref (data_mvnd);
@@ -161,12 +161,12 @@ ncm_data_gauss_cov_mvnd_free (NcmDataGaussCovMVND *data_mvnd)
 /**
  * ncm_data_gauss_cov_mvnd_clear:
  * @data_mvnd: a #NcmDataGaussCovMVND
- * 
- * If @data_mvnd is different from NULL, decreases the reference count of 
+ *
+ * If @data_mvnd is different from NULL, decreases the reference count of
  * @data_mvnd by one and sets @data_mvnd to NULL.
- * 
+ *
  */
-void 
+void
 ncm_data_gauss_cov_mvnd_clear (NcmDataGaussCovMVND **data_mvnd)
 {
   g_clear_object (data_mvnd);
@@ -181,31 +181,38 @@ ncm_data_gauss_cov_mvnd_clear (NcmDataGaussCovMVND **data_mvnd)
  * @mean_min: minimum mean $\mu_i$
  * @mean_max: maximum mean $\mu_i$
  * @rng: a #NcmRNG
- * 
- * Generates using @rng the mean and correlation matrix using 
+ *
+ * Generates using @rng the mean and correlation matrix using
  * the parameters above.
- * 
+ *
  */
-void 
+void
 ncm_data_gauss_cov_mvnd_gen_cov_mean (NcmDataGaussCovMVND *data_mvnd, const gdouble sigma_min, const gdouble sigma_max, const gdouble cor_level, const gdouble mean_min, const gdouble mean_max, NcmRNG *rng)
 {
   NcmDataGaussCov *gcov = NCM_DATA_GAUSS_COV (data_mvnd);
+  NcmVector *y   = ncm_data_gauss_cov_peek_mean (gcov);
+  NcmMatrix *cov = ncm_data_gauss_cov_peek_cov (gcov);
+  const guint np = ncm_data_gauss_cov_get_size (gcov);
   gint i;
 
   g_assert_cmpfloat (mean_min, <=, mean_max);
+  g_assert_cmpint (np, >, 0);
+  g_assert (y != NULL);
+  g_assert (cov != NULL);
 
-  ncm_matrix_fill_rand_cov (gcov->cov, sigma_min, sigma_max, cor_level, rng);
+  ncm_matrix_fill_rand_cov (cov, sigma_min, sigma_max, cor_level, rng);
 
   if (mean_min == mean_max)
   {
-    ncm_vector_set_all (gcov->y, mean_min);
+    ncm_vector_set_all (y, mean_min);
   }
   else
   {
-    for (i = 0; i < gcov->np; i++)
+    for (i = 0; i < np; i++)
     {
       const gdouble mean = ncm_rng_uniform_gen (rng, mean_min, mean_max);
-      ncm_vector_set (gcov->y, i, mean);
+
+      ncm_vector_set (y, i, mean);
     }
   }
 
@@ -217,20 +224,24 @@ ncm_data_gauss_cov_mvnd_gen_cov_mean (NcmDataGaussCovMVND *data_mvnd, const gdou
  * @data_mvnd: a #NcmDataGaussCovMVND
  * @mean: a #NcmVector
  * @cov: a #NcmMatrix
- * 
+ *
  * Sets the mean and covariance of @data_mvnd.
- * 
+ *
  */
-void 
+void
 ncm_data_gauss_cov_mvnd_set_cov_mean (NcmDataGaussCovMVND *data_mvnd, NcmVector *mean, NcmMatrix *cov)
 {
   NcmDataGaussCov *gcov = NCM_DATA_GAUSS_COV (data_mvnd);
+  NcmVector *cy         = ncm_data_gauss_cov_peek_mean (gcov);
+  NcmMatrix *ccov       = ncm_data_gauss_cov_peek_cov (gcov);
 
   g_assert_cmpuint (ncm_vector_len (mean), ==, ncm_matrix_nrows (cov));
   g_assert_cmpuint (ncm_vector_len (mean), ==, ncm_matrix_ncols (cov));
+  g_assert (cy != NULL);
+  g_assert (cov != NULL);
 
-  ncm_matrix_memcpy (gcov->cov, cov);
-  ncm_vector_memcpy (gcov->y, mean);
+  ncm_matrix_memcpy (ccov, cov);
+  ncm_vector_memcpy (cy, mean);
 
   ncm_data_set_init (NCM_DATA (gcov), TRUE);
 }
@@ -238,16 +249,18 @@ ncm_data_gauss_cov_mvnd_set_cov_mean (NcmDataGaussCovMVND *data_mvnd, NcmVector
 /**
  * ncm_data_gauss_cov_mvnd_peek_mean:
  * @data_mvnd: a #NcmDataGaussCovMVND
- * 
+ *
  * Peeks current mean vector.
- * 
+ *
  * Returns: (transfer none): the current mean vector.
- */ 
+ */
 NcmVector *
 ncm_data_gauss_cov_mvnd_peek_mean (NcmDataGaussCovMVND *data_mvnd)
 {
   NcmDataGaussCov *gcov = NCM_DATA_GAUSS_COV (data_mvnd);
-  return gcov->y;
+  NcmVector *y         = ncm_data_gauss_cov_peek_mean (gcov);
+
+  return y;
 }
 
 /**
@@ -258,24 +271,25 @@ ncm_data_gauss_cov_mvnd_peek_mean (NcmDataGaussCovMVND *data_mvnd)
  * @bound: (scope call) (allow-none): a NcmDataGaussCovMVNDBound
  * @rng: a #NcmRNG
  * @N: (out): number of realizations necessary to generate a valid one
- * 
+ *
  * Generates one realization of the MVND. If @bound is not NULL,
  * generates realizations untill @bound returns TRUE.
- * 
+ *
  * Returns: (transfer none): a #NcmVector (should not be modified)
- */ 
+ */
 NcmVector *
 ncm_data_gauss_cov_mvnd_gen (NcmDataGaussCovMVND *data_mvnd, NcmMSet *mset, gpointer obj, NcmDataGaussCovMVNDBound bound, NcmRNG *rng, gulong *N)
 {
-  NcmDataGaussCov *cov = NCM_DATA_GAUSS_COV (data_mvnd);
-  NcmData *data        = NCM_DATA (data_mvnd);
-  gulong maxiter       = 100000000;
+  NcmDataGaussCov *gcov = NCM_DATA_GAUSS_COV (data_mvnd);
+  NcmData *data         = NCM_DATA (data_mvnd);
+  NcmVector *y          = ncm_data_gauss_cov_peek_mean (gcov);
+  gulong maxiter        = 100000000;
 
   N[0] = 0;
+
   if (bound != NULL)
   {
-    do
-    {
+    do {
       ncm_data_resample (data, mset, rng);
       N[0]++;
 
@@ -284,13 +298,14 @@ ncm_data_gauss_cov_mvnd_gen (NcmDataGaussCovMVND *data_mvnd, NcmMSet *mset, gpoi
         g_error ("ncm_data_gauss_cov_mvnd_gen: too many interations, cannot find a valid realization!");
         break;
       }
-
-    } while (!bound (obj, cov->y));
+    } while (!bound (obj, y));
   }
   else
+  {
     ncm_data_resample (data, mset, rng);
+  }
 
-  return cov->y;
+  return y;
 }
 
 /**
@@ -303,22 +318,22 @@ ncm_data_gauss_cov_mvnd_gen (NcmDataGaussCovMVND *data_mvnd, NcmMSet *mset, gpoi
  * @Nin: (inout): number of realizations accepted
  * @reltol: relative tolerance
  * @rng: a #NcmRNG
- * 
- * Estimate the ratio between accepted realizations and total number 
+ *
+ * Estimate the ratio between accepted realizations and total number
  * of realizations. The variable @reltol controls the relative tolerance
  * on the ratio estimate. The variables @N and @Nin can be used to inform
  * previous number of realizations.
  *
  * Returns: estimated ratio
- */ 
-gdouble 
+ */
+gdouble
 ncm_data_gauss_cov_mvnd_est_ratio (NcmDataGaussCovMVND *data_mvnd, NcmMSet *mset, gpointer obj, NcmDataGaussCovMVNDBound bound, gulong *N, gulong *Nin, const gdouble reltol, NcmRNG *rng)
 {
   gulong NN   = (N   != NULL) ? *N   : 0;
   gulong NNin = (Nin != NULL) ? *Nin : 0;
   gdouble ratio;
   gulong maxiter = 100000000;
-  glong miniter  = MIN (10000, (glong)(1.0 / reltol));
+  glong miniter  = MIN (10000, (glong) (1.0 / reltol));
 
   while (TRUE)
   {
@@ -345,6 +360,7 @@ ncm_data_gauss_cov_mvnd_est_ratio (NcmDataGaussCovMVND *data_mvnd, NcmMSet *mset
 
   if (N != NULL)
     *N = NN;
+
   if (Nin != NULL)
     *Nin = NNin;
 
@@ -354,15 +370,21 @@ ncm_data_gauss_cov_mvnd_est_ratio (NcmDataGaussCovMVND *data_mvnd, NcmMSet *mset
 /**
  * ncm_data_gauss_cov_mvnd_log_info:
  * @data_mvnd: a #NcmDataGaussCovMVND
- * 
+ *
  * Logs mean and covariance matrix.
- * 
- */ 
-void 
+ *
+ */
+void
 ncm_data_gauss_cov_mvnd_log_info (NcmDataGaussCovMVND *data_mvnd)
 {
   NcmDataGaussCov *gcov = NCM_DATA_GAUSS_COV (data_mvnd);
+  NcmVector *y         = ncm_data_gauss_cov_peek_mean (gcov);
+  NcmMatrix *cov       = ncm_data_gauss_cov_peek_cov (gcov);
 
-  ncm_vector_log_vals (gcov->y,   "# NcmDataGaussCovMVND data mean: ", "% 12.5g", TRUE);
-  ncm_matrix_log_vals (gcov->cov, "# NcmDataGaussCovMVND data cov: ", "% 12.5g");
+  g_assert (y != NULL);
+  g_assert (cov != NULL);
+
+  ncm_vector_log_vals (y,   "# NcmDataGaussCovMVND data mean: ", "% 12.5g", TRUE);
+  ncm_matrix_log_vals (cov, "# NcmDataGaussCovMVND data cov: ", "% 12.5g");
 }
+
