@@ -8,17 +8,17 @@
 /*
  * numcosmo
  * Copyright (C) 2012 Sandro Dias Pinto Vitenti <vitenti@uel.br>
- * 
+ *
  * numcosmo is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
- * 
+ *
  * numcosmo is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License along
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
@@ -29,7 +29,7 @@
  * @short_description: Gaussian data -- diagonal covariance provided.
  *
  * Gaussian distribution which uses a diagonal covariance matrix as input.
- * 
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -57,19 +57,33 @@ enum
   PROP_SIZE,
 };
 
-G_DEFINE_ABSTRACT_TYPE (NcmDataGaussDiag, ncm_data_gauss_diag, NCM_TYPE_DATA);
+typedef struct _NcmDataGaussDiagPrivate
+{
+  guint np;
+  NcmVector *y;
+  NcmVector *v;
+  NcmVector *sigma;
+  NcmVector *weight;
+  gdouble wt;
+  gboolean prepared_w;
+  gboolean wmean;
+} NcmDataGaussDiagPrivate;
+
+G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (NcmDataGaussDiag, ncm_data_gauss_diag, NCM_TYPE_DATA);
 
 static void
 ncm_data_gauss_diag_init (NcmDataGaussDiag *diag)
 {
-  diag->np         = 0;
-  diag->y          = NULL;
-  diag->v          = NULL;
-  diag->sigma      = NULL;
-  diag->weight     = NULL;
-  diag->wt         = 0.0;
-  diag->prepared_w = FALSE;
-  diag->wmean      = FALSE;
+  NcmDataGaussDiagPrivate * const self = ncm_data_gauss_diag_get_instance_private (diag);
+
+  self->np         = 0;
+  self->y          = NULL;
+  self->v          = NULL;
+  self->sigma      = NULL;
+  self->weight     = NULL;
+  self->wt         = 0.0;
+  self->prepared_w = FALSE;
+  self->wmean      = FALSE;
 }
 
 static void
@@ -77,13 +91,14 @@ _ncm_data_gauss_diag_constructed (GObject *object)
 {
   /* Chain up : start */
   G_OBJECT_CLASS (ncm_data_gauss_diag_parent_class)->constructed (object);
-
 }
 
 static void
 ncm_data_gauss_diag_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
   NcmDataGaussDiag *diag = NCM_DATA_GAUSS_DIAG (object);
+  NcmDataGaussDiagPrivate * const self = ncm_data_gauss_diag_get_instance_private (diag);
+
   g_return_if_fail (NCM_IS_DATA_GAUSS_DIAG (object));
 
   switch (prop_id)
@@ -92,13 +107,13 @@ ncm_data_gauss_diag_set_property (GObject *object, guint prop_id, const GValue *
       ncm_data_gauss_diag_set_size (diag, g_value_get_uint (value));
       break;
     case PROP_WMEAN:
-      diag->wmean = g_value_get_boolean (value);
+      self->wmean = g_value_get_boolean (value);
       break;
     case PROP_MEAN:
-      ncm_vector_substitute (&diag->y, g_value_get_object (value), TRUE);
+      ncm_vector_substitute (&self->y, g_value_get_object (value), TRUE);
       break;
     case PROP_SIGMA:
-      ncm_vector_substitute (&diag->sigma, g_value_get_object (value), TRUE);
+      ncm_vector_substitute (&self->sigma, g_value_get_object (value), TRUE);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -110,6 +125,8 @@ static void
 ncm_data_gauss_diag_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
   NcmDataGaussDiag *diag = NCM_DATA_GAUSS_DIAG (object);
+  NcmDataGaussDiagPrivate * const self = ncm_data_gauss_diag_get_instance_private (diag);
+
   g_return_if_fail (NCM_IS_DATA_GAUSS_DIAG (object));
 
   switch (prop_id)
@@ -118,13 +135,13 @@ ncm_data_gauss_diag_get_property (GObject *object, guint prop_id, GValue *value,
       g_value_set_uint (value, ncm_data_gauss_diag_get_size (diag));
       break;
     case PROP_WMEAN:
-      g_value_set_boolean (value, diag->wmean);
+      g_value_set_boolean (value, self->wmean);
       break;
     case PROP_MEAN:
-      g_value_set_object (value, diag->y);
+      g_value_set_object (value, self->y);
       break;
     case PROP_SIGMA:
-      g_value_set_object (value, diag->sigma);
+      g_value_set_object (value, self->sigma);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -136,11 +153,12 @@ static void
 ncm_data_gauss_diag_dispose (GObject *object)
 {
   NcmDataGaussDiag *diag = NCM_DATA_GAUSS_DIAG (object);
+  NcmDataGaussDiagPrivate * const self = ncm_data_gauss_diag_get_instance_private (diag);
 
-  ncm_vector_clear (&diag->y);
-  ncm_vector_clear (&diag->v);
-  ncm_vector_clear (&diag->sigma);
-  ncm_vector_clear (&diag->weight);
+  ncm_vector_clear (&self->y);
+  ncm_vector_clear (&self->v);
+  ncm_vector_clear (&self->sigma);
+  ncm_vector_clear (&self->weight);
 
   /* Chain up : end */
   G_OBJECT_CLASS (ncm_data_gauss_diag_parent_class)->dispose (object);
@@ -149,13 +167,13 @@ ncm_data_gauss_diag_dispose (GObject *object)
 static void
 ncm_data_gauss_diag_finalize (GObject *object)
 {
-  
   /* Chain up : end */
   G_OBJECT_CLASS (ncm_data_gauss_diag_parent_class)->finalize (object);
 }
 
 static guint _ncm_data_gauss_diag_get_length (NcmData *data);
 static guint _ncm_data_gauss_diag_get_dof (NcmData *data);
+
 /* static void _ncm_data_gauss_diag_begin (NcmData *data); */
 static void _ncm_data_gauss_diag_resample (NcmData *data, NcmMSet *mset, NcmRNG *rng);
 static void _ncm_data_gauss_diag_m2lnL_val (NcmData *data, NcmMSet *mset, gdouble *m2lnL);
@@ -169,8 +187,8 @@ static guint _ncm_data_gauss_diag_get_size (NcmDataGaussDiag *diag);
 static void
 ncm_data_gauss_diag_class_init (NcmDataGaussDiagClass *klass)
 {
-  GObjectClass* object_class     = G_OBJECT_CLASS (klass);
-  NcmDataClass *data_class       = NCM_DATA_CLASS (klass);
+  GObjectClass *object_class              = G_OBJECT_CLASS (klass);
+  NcmDataClass *data_class                = NCM_DATA_CLASS (klass);
   NcmDataGaussDiagClass *gauss_diag_class = NCM_DATA_GAUSS_DIAG_CLASS (klass);
 
   object_class->constructed  = &_ncm_data_gauss_diag_constructed;
@@ -210,15 +228,15 @@ ncm_data_gauss_diag_class_init (NcmDataGaussDiagClass *klass)
                                                         "Data standard deviation",
                                                         NCM_TYPE_VECTOR,
                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
-  
-  data_class->bootstrap        = TRUE;
-  data_class->get_length       = &_ncm_data_gauss_diag_get_length;
-  data_class->get_dof          = &_ncm_data_gauss_diag_get_dof;
-  data_class->resample         = &_ncm_data_gauss_diag_resample;
-  data_class->m2lnL_val        = &_ncm_data_gauss_diag_m2lnL_val;
-  data_class->leastsquares_f   = &_ncm_data_gauss_diag_leastsquares_f;
-  data_class->mean_vector      = &_ncm_data_gauss_diag_mean_vector;
-  data_class->inv_cov_UH       = &_ncm_data_gauss_diag_inv_cov_UH;
+
+  data_class->bootstrap      = TRUE;
+  data_class->get_length     = &_ncm_data_gauss_diag_get_length;
+  data_class->get_dof        = &_ncm_data_gauss_diag_get_dof;
+  data_class->resample       = &_ncm_data_gauss_diag_resample;
+  data_class->m2lnL_val      = &_ncm_data_gauss_diag_m2lnL_val;
+  data_class->leastsquares_f = &_ncm_data_gauss_diag_leastsquares_f;
+  data_class->mean_vector    = &_ncm_data_gauss_diag_mean_vector;
+  data_class->inv_cov_UH     = &_ncm_data_gauss_diag_inv_cov_UH;
 
   gauss_diag_class->mean_func  = NULL;
   gauss_diag_class->sigma_func = NULL;
@@ -226,18 +244,25 @@ ncm_data_gauss_diag_class_init (NcmDataGaussDiagClass *klass)
   gauss_diag_class->get_size   = &_ncm_data_gauss_diag_get_size;
 }
 
-static guint 
-_ncm_data_gauss_diag_get_length (NcmData *data) 
-{ 
-  return NCM_DATA_GAUSS_DIAG (data)->np; 
+static guint
+_ncm_data_gauss_diag_get_length (NcmData *data)
+{
+  NcmDataGaussDiag *diag = NCM_DATA_GAUSS_DIAG (data);
+  NcmDataGaussDiagPrivate * const self = ncm_data_gauss_diag_get_instance_private (diag);
+
+  return self->np;
 }
 
-static guint 
-_ncm_data_gauss_diag_get_dof (NcmData *data) 
-{ 
-  guint dof = NCM_DATA_GAUSS_DIAG (data)->np;
-  if (NCM_DATA_GAUSS_DIAG (data)->wmean && dof > 0)
+static guint
+_ncm_data_gauss_diag_get_dof (NcmData *data)
+{
+  NcmDataGaussDiag *diag = NCM_DATA_GAUSS_DIAG (data);
+  NcmDataGaussDiagPrivate * const self = ncm_data_gauss_diag_get_instance_private (diag);
+  guint dof = self->np;
+
+  if (self->wmean && (dof > 0))
     dof--;
+
   return dof;
 }
 
@@ -245,99 +270,112 @@ static void
 _ncm_data_gauss_prepare_weight (NcmData *data)
 {
   NcmDataGaussDiag *diag = NCM_DATA_GAUSS_DIAG (data);
+  NcmDataGaussDiagPrivate * const self = ncm_data_gauss_diag_get_instance_private (diag);
   guint i;
 
-  if (diag->weight == NULL)
-    diag->weight = ncm_vector_new (diag->np);
+  if (self->weight == NULL)
+    self->weight = ncm_vector_new (self->np);
 
-  diag->wt = 0.0;
-  for (i = 0; i < diag->np; i++)
+  self->wt = 0.0;
+
+  for (i = 0; i < self->np; i++)
   {
-    const gdouble sigma_i = ncm_vector_get (diag->sigma, i);
-    const gdouble w_i = 1.0 / (sigma_i * sigma_i);
-    ncm_vector_set (diag->weight, i, w_i);
-    diag->wt += w_i;
+    const gdouble sigma_i = ncm_vector_get (self->sigma, i);
+    const gdouble w_i     = 1.0 / (sigma_i * sigma_i);
+
+    ncm_vector_set (self->weight, i, w_i);
+    self->wt += w_i;
   }
 
-  diag->prepared_w = TRUE;
+  self->prepared_w = TRUE;
 }
 
 static void
 _ncm_data_gauss_diag_resample (NcmData *data, NcmMSet *mset, NcmRNG *rng)
 {
-  NcmDataGaussDiag *diag = NCM_DATA_GAUSS_DIAG (data);
+  NcmDataGaussDiag *diag                  = NCM_DATA_GAUSS_DIAG (data);
+  NcmDataGaussDiagPrivate * const self    = ncm_data_gauss_diag_get_instance_private (diag);
   NcmDataGaussDiagClass *gauss_diag_class = NCM_DATA_GAUSS_DIAG_GET_CLASS (diag);
   guint i;
 
   if (gauss_diag_class->sigma_func != NULL)
-    gauss_diag_class->sigma_func (diag, mset, diag->sigma);
+    gauss_diag_class->sigma_func (diag, mset, self->sigma);
+
+  gauss_diag_class->mean_func (diag, mset, self->y);
 
-  gauss_diag_class->mean_func (diag, mset, diag->y); 
-  
   ncm_rng_lock (rng);
-  for (i = 0; i < diag->np; i++)
+
+  for (i = 0; i < self->np; i++)
   {
-    const gdouble sigma_i = ncm_vector_get (diag->sigma, i);
-    const gdouble u_i = gsl_ran_gaussian_ziggurat (rng->r, sigma_i);
-    ncm_vector_subfrom (diag->y, i, u_i);
+    const gdouble sigma_i = ncm_vector_get (self->sigma, i);
+    const gdouble u_i     = gsl_ran_gaussian_ziggurat (rng->r, sigma_i);
+
+    ncm_vector_subfrom (self->y, i, u_i);
   }
+
   ncm_rng_unlock (rng);
 }
 
 static void
 _ncm_data_gauss_diag_m2lnL_val (NcmData *data, NcmMSet *mset, gdouble *m2lnL)
 {
-  NcmDataGaussDiag *diag = NCM_DATA_GAUSS_DIAG (data);
+  NcmDataGaussDiag *diag                  = NCM_DATA_GAUSS_DIAG (data);
+  NcmDataGaussDiagPrivate * const self    = ncm_data_gauss_diag_get_instance_private (diag);
   NcmDataGaussDiagClass *gauss_diag_class = NCM_DATA_GAUSS_DIAG_GET_CLASS (diag);
-  gboolean sigma_update = FALSE;
+  gboolean sigma_update                   = FALSE;
   guint i;
 
   *m2lnL = 0.0;
 
   if (gauss_diag_class->sigma_func != NULL)
-    sigma_update = gauss_diag_class->sigma_func (diag, mset, diag->sigma);
+    sigma_update = gauss_diag_class->sigma_func (diag, mset, self->sigma);
 
-  gauss_diag_class->mean_func (diag, mset, diag->v);
+  gauss_diag_class->mean_func (diag, mset, self->v);
 
-  if (diag->wmean)
+  if (self->wmean)
   {
-    if (sigma_update || !diag->prepared_w)
+    if (sigma_update || !self->prepared_w)
       _ncm_data_gauss_prepare_weight (data);
 
     if (!ncm_data_bootstrap_enabled (data))
     {
       gdouble tmp = 0.0;
 
-      for (i = 0; i < diag->np; i++)
+      for (i = 0; i < self->np; i++)
       {
-        const gdouble y_i = ncm_vector_get (diag->y, i);
-        const gdouble yt_i = ncm_vector_get (diag->v, i);
-        const gdouble r_i = (yt_i - y_i);
-        const gdouble w_i = ncm_vector_get (diag->weight, i);
+        const gdouble y_i  = ncm_vector_get (self->y, i);
+        const gdouble yt_i = ncm_vector_get (self->v, i);
+        const gdouble r_i  = (yt_i - y_i);
+        const gdouble w_i  = ncm_vector_get (self->weight, i);
         const gdouble r_i2 = r_i * r_i;
+
         *m2lnL += r_i2 * w_i;
-        tmp += r_i * w_i;
+        tmp    += r_i * w_i;
       }
-      *m2lnL -= tmp * tmp / diag->wt;
+
+      *m2lnL -= tmp * tmp / self->wt;
     }
     else
     {
-      const guint bsize = ncm_bootstrap_get_bsize (data->bstrap);
-      gdouble tmp = 0.0;
-      gdouble wt = 0.0;
-      
+      NcmBootstrap *bstrap = ncm_data_peek_bootstrap (data);
+      const guint bsize = ncm_bootstrap_get_bsize (bstrap);
+      gdouble tmp       = 0.0;
+      gdouble wt        = 0.0;
+
       for (i = 0; i < bsize; i++)
       {
-        guint k = ncm_bootstrap_get (data->bstrap, i);
-        const gdouble y_k = ncm_vector_get (diag->y, k);
-        const gdouble yt_k = ncm_vector_get (diag->v, k);
-        const gdouble r_k = (yt_k - y_k);
-        const gdouble w_k = ncm_vector_get (diag->weight, k);
+        guint k            = ncm_bootstrap_get (bstrap, i);
+        const gdouble y_k  = ncm_vector_get (self->y, k);
+        const gdouble yt_k = ncm_vector_get (self->v, k);
+        const gdouble r_k  = (yt_k - y_k);
+        const gdouble w_k  = ncm_vector_get (self->weight, k);
         const gdouble r_k2 = r_k * r_k;
+
         *m2lnL += r_k2 * w_k;
-        tmp += r_k * w_k;
-        wt += w_k;
+        tmp    += r_k * w_k;
+        wt     += w_k;
       }
+
       *m2lnL -= tmp * tmp / wt;
     }
   }
@@ -345,25 +383,29 @@ _ncm_data_gauss_diag_m2lnL_val (NcmData *data, NcmMSet *mset, gdouble *m2lnL)
   {
     if (!ncm_data_bootstrap_enabled (data))
     {
-      for (i = 0; i < diag->np; i++)
+      for (i = 0; i < self->np; i++)
       {
-        const gdouble y_i = ncm_vector_get (diag->y, i);
-        const gdouble yt_i = ncm_vector_get (diag->v, i);
-        const gdouble sigma_i = ncm_vector_get (diag->sigma, i);
-        const gdouble r_i = (yt_i - y_i) / sigma_i;
+        const gdouble y_i     = ncm_vector_get (self->y, i);
+        const gdouble yt_i    = ncm_vector_get (self->v, i);
+        const gdouble sigma_i = ncm_vector_get (self->sigma, i);
+        const gdouble r_i     = (yt_i - y_i) / sigma_i;
+
         *m2lnL += r_i * r_i;
       }
     }
     else
     {
-      const guint bsize = ncm_bootstrap_get_bsize (data->bstrap);
+      NcmBootstrap *bstrap = ncm_data_peek_bootstrap (data);
+      const guint bsize = ncm_bootstrap_get_bsize (bstrap);
+
       for (i = 0; i < bsize; i++)
       {
-        guint k = ncm_bootstrap_get (data->bstrap, i);
-        const gdouble y_k = ncm_vector_get (diag->y, k);
-        const gdouble yt_k = ncm_vector_get (diag->v, k);
-        const gdouble sigma_k = ncm_vector_get (diag->sigma, k);
-        const gdouble r_k = (yt_k - y_k) / sigma_k;
+        guint k               = ncm_bootstrap_get (bstrap, i);
+        const gdouble y_k     = ncm_vector_get (self->y, k);
+        const gdouble yt_k    = ncm_vector_get (self->v, k);
+        const gdouble sigma_k = ncm_vector_get (self->sigma, k);
+        const gdouble r_k     = (yt_k - y_k) / sigma_k;
+
         *m2lnL += r_k * r_k;
       }
     }
@@ -373,48 +415,52 @@ _ncm_data_gauss_diag_m2lnL_val (NcmData *data, NcmMSet *mset, gdouble *m2lnL)
 static void
 _ncm_data_gauss_diag_leastsquares_f (NcmData *data, NcmMSet *mset, NcmVector *v)
 {
-  NcmDataGaussDiag *diag = NCM_DATA_GAUSS_DIAG (data);
+  NcmDataGaussDiag *diag                  = NCM_DATA_GAUSS_DIAG (data);
+  NcmDataGaussDiagPrivate * const self    = ncm_data_gauss_diag_get_instance_private (diag);
   NcmDataGaussDiagClass *gauss_diag_class = NCM_DATA_GAUSS_DIAG_GET_CLASS (diag);
-  gboolean sigma_update = FALSE;
+  gboolean sigma_update                   = FALSE;
   guint i;
 
   if (ncm_data_bootstrap_enabled (data))
     g_error ("NcmDataGaussDiag: does not support bootstrap with least squares");
-  
+
   if (gauss_diag_class->sigma_func != NULL)
-    sigma_update = gauss_diag_class->sigma_func (diag, mset, diag->sigma);
+    sigma_update = gauss_diag_class->sigma_func (diag, mset, self->sigma);
 
   gauss_diag_class->mean_func (diag, mset, v);
 
-  if (diag->wmean)
+  if (self->wmean)
   {
     gdouble wmean;
-    if (sigma_update || !diag->prepared_w)
+
+    if (sigma_update || !self->prepared_w)
       _ncm_data_gauss_prepare_weight (data);
 
-    ncm_vector_sub (v, diag->y);
+    ncm_vector_sub (v, self->y);
 
-    wmean = gsl_stats_wmean (ncm_vector_gsl (diag->weight)->data,
-                             ncm_vector_gsl (diag->weight)->stride,
+    wmean = gsl_stats_wmean (ncm_vector_gsl (self->weight)->data,
+                             ncm_vector_gsl (self->weight)->stride,
                              ncm_vector_gsl (v)->data,
                              ncm_vector_gsl (v)->stride,
                              ncm_vector_gsl (v)->size);
 
-    for (i = 0; i < diag->np; i++)
+    for (i = 0; i < self->np; i++)
     {
-      const gdouble sigma_i = ncm_vector_get (diag->sigma, i);
-      const gdouble v_i = ncm_vector_get (v, i);
+      const gdouble sigma_i = ncm_vector_get (self->sigma, i);
+      const gdouble v_i     = ncm_vector_get (v, i);
+
       ncm_vector_set (v, i, (v_i - wmean) / sigma_i);
     }
   }
   else
   {
-    for (i = 0; i < diag->np; i++)
+    for (i = 0; i < self->np; i++)
     {
-      const gdouble y_i = ncm_vector_get (diag->y, i);
-      const gdouble yt_i = ncm_vector_get (v, i);
-      const gdouble sigma_i = ncm_vector_get (diag->sigma, i);
-      const gdouble r_i = (yt_i - y_i) / sigma_i;
+      const gdouble y_i     = ncm_vector_get (self->y, i);
+      const gdouble yt_i    = ncm_vector_get (v, i);
+      const gdouble sigma_i = ncm_vector_get (self->sigma, i);
+      const gdouble r_i     = (yt_i - y_i) / sigma_i;
+
       ncm_vector_set (v, i, r_i);
     }
   }
@@ -433,54 +479,66 @@ static void
 _ncm_data_gauss_diag_inv_cov_UH (NcmData *data, NcmMSet *mset, NcmMatrix *H)
 {
   NcmDataGaussDiag *diag                  = NCM_DATA_GAUSS_DIAG (data);
+  NcmDataGaussDiagPrivate * const self    = ncm_data_gauss_diag_get_instance_private (diag);
   NcmDataGaussDiagClass *gauss_diag_class = NCM_DATA_GAUSS_DIAG_GET_CLASS (diag);
   guint i;
 
   if (ncm_data_bootstrap_enabled (data))
     g_error ("NcmDataGaussDiag: does not support bootstrap with least squares");
-  
+
   if (gauss_diag_class->sigma_func != NULL)
-    gauss_diag_class->sigma_func (diag, mset, diag->sigma);
-  
-  for (i = 0; i < diag->np; i++)
+    gauss_diag_class->sigma_func (diag, mset, self->sigma);
+
+  for (i = 0; i < self->np; i++)
   {
-    const gdouble sigma_i = ncm_vector_get (diag->sigma, i);
+    const gdouble sigma_i = ncm_vector_get (self->sigma, i);
+
     ncm_matrix_mul_col (H, i, 1.0 / sigma_i);
   }
 }
 
-static void 
+static void
 _ncm_data_gauss_diag_set_size (NcmDataGaussDiag *diag, guint np)
 {
   NcmData *data = NCM_DATA (diag);
-  if ((np == 0) || (np != diag->np))
+  NcmDataGaussDiagPrivate * const self = ncm_data_gauss_diag_get_instance_private (diag);
+
+  if ((np == 0) || (np != self->np))
   {
-    diag->np = 0;
-    ncm_vector_clear (&diag->y);
-    ncm_vector_clear (&diag->v);
-    ncm_vector_clear (&diag->sigma);
-    ncm_vector_clear (&diag->weight);
-    data->init = FALSE;
+    self->np = 0;
+    ncm_vector_clear (&self->y);
+    ncm_vector_clear (&self->v);
+    ncm_vector_clear (&self->sigma);
+    ncm_vector_clear (&self->weight);
+
+    ncm_data_set_init (data, FALSE);
   }
-  if ((np != 0) && (np != diag->np))
+
+  if ((np != 0) && (np != self->np))
   {
-    diag->np    = np;
-    diag->y     = ncm_vector_new (diag->np);
-    diag->v     = ncm_vector_new (diag->np);
-    diag->sigma = ncm_vector_new (diag->np);
+    NcmBootstrap *bstrap = ncm_data_peek_bootstrap (data);
+
+    self->np    = np;
+    self->y     = ncm_vector_new (self->np);
+    self->v     = ncm_vector_new (self->np);
+    self->sigma = ncm_vector_new (self->np);
+
     if (ncm_data_bootstrap_enabled (data))
     {
-      ncm_bootstrap_set_fsize (data->bstrap, np);
-      ncm_bootstrap_set_bsize (data->bstrap, np);
+      ncm_bootstrap_set_fsize (bstrap, np);
+      ncm_bootstrap_set_bsize (bstrap, np);
     }
-    data->init = FALSE;
+
+    ncm_data_set_init (data, FALSE);
   }
 }
 
-static guint 
+static guint
 _ncm_data_gauss_diag_get_size (NcmDataGaussDiag *diag)
 {
-  return diag->np;
+  NcmDataGaussDiagPrivate * const self = ncm_data_gauss_diag_get_instance_private (diag);
+
+  return self->np;
 }
 
 /**
@@ -489,9 +547,9 @@ _ncm_data_gauss_diag_get_size (NcmDataGaussDiag *diag)
  * @np: data size.
  *
  * Sets the data size to @np.
- * 
+ *
  */
-void 
+void
 ncm_data_gauss_diag_set_size (NcmDataGaussDiag *diag, guint np)
 {
   NCM_DATA_GAUSS_DIAG_GET_CLASS (diag)->set_size (diag, np);
@@ -502,12 +560,40 @@ ncm_data_gauss_diag_set_size (NcmDataGaussDiag *diag, guint np)
  * @diag: a #NcmDataGaussDiag
  *
  * Gets the data size.
- * 
+ *
  * Returns: Data size.
- * 
+ *
  */
-guint 
+guint
 ncm_data_gauss_diag_get_size (NcmDataGaussDiag *diag)
 {
   return NCM_DATA_GAUSS_DIAG_GET_CLASS (diag)->get_size (diag);
 }
+
+/**
+ * ncm_data_gauss_diag_peek_mean:
+ * @diag: a #NcmDataGaussDiag
+ *
+ * Returns: (transfer none): the current data mean #NcmVector.
+ */
+NcmVector *
+ncm_data_gauss_diag_peek_mean (NcmDataGaussDiag *diag)
+{
+  NcmDataGaussDiagPrivate * const self = ncm_data_gauss_diag_get_instance_private (diag);
+
+  return self->y;
+}
+
+/**
+ * ncm_data_gauss_diag_peek_std:
+ * @diag: a #NcmDataGaussDiag
+ *
+ * Returns: (transfer none): the current data standard deviation #NcmVector.
+ */
+NcmVector *
+ncm_data_gauss_diag_peek_std (NcmDataGaussDiag *diag)
+{
+  NcmDataGaussDiagPrivate * const self = ncm_data_gauss_diag_get_instance_private (diag);
+
+  return self->sigma;
+}
