@@ -28,7 +28,7 @@
  * @title: NcmDataset
  * @short_description: A set of NcmData objects
  *
- * FIXME
+ * This Class define a set of NcmData objects.
  *
  */
 
@@ -49,6 +49,16 @@ enum
   PROP_SIZE,
 };
 
+struct _NcmDataset
+{
+  /*< private >*/
+  GObject parent_instance;
+  NcmObjArray *oa;
+  NcmDatasetBStrapType bstype;
+  GArray *data_prob;
+  GArray *bstrap;
+};
+
 G_DEFINE_TYPE (NcmDataset, ncm_dataset, G_TYPE_OBJECT);
 
 #define _NCM_DATASET_INITIAL_ALLOC 10
@@ -66,6 +76,7 @@ static void
 ncm_dataset_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
   NcmDataset *dset = NCM_DATASET (object);
+
   g_return_if_fail (NCM_IS_DATASET (object));
 
   switch (prop_id)
@@ -86,6 +97,7 @@ static void
 ncm_dataset_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
   NcmDataset *dset = NCM_DATASET (object);
+
   g_return_if_fail (NCM_IS_DATASET (object));
 
   switch (prop_id)
@@ -114,6 +126,7 @@ ncm_dataset_dispose (GObject *object)
     g_array_unref (dset->data_prob);
     dset->data_prob = NULL;
   }
+
   if (dset->bstrap != NULL)
   {
     g_array_unref (dset->bstrap);
@@ -127,7 +140,6 @@ ncm_dataset_dispose (GObject *object)
 static void
 ncm_dataset_finalize (GObject *object)
 {
-
   /* Chain up : end */
   G_OBJECT_CLASS (ncm_dataset_parent_class)->finalize (object);
 }
@@ -135,7 +147,7 @@ ncm_dataset_finalize (GObject *object)
 static void
 ncm_dataset_class_init (NcmDatasetClass *klass)
 {
-  GObjectClass* object_class = G_OBJECT_CLASS (klass);
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
   object_class->set_property = &ncm_dataset_set_property;
   object_class->get_property = &ncm_dataset_get_property;
@@ -155,6 +167,7 @@ ncm_dataset_class_init (NcmDatasetClass *klass)
                                                       "Bootstrap type",
                                                       NCM_TYPE_DATASET_BSTRAP_TYPE, NCM_DATASET_BSTRAP_DISABLE,
                                                       G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+
   /**
    * NcmData:data-array:
    *
@@ -181,6 +194,7 @@ NcmDataset *
 ncm_dataset_new (void)
 {
   NcmDataset *dset = g_object_new (NCM_TYPE_DATASET, NULL);
+
   return dset;
 }
 
@@ -190,7 +204,7 @@ ncm_dataset_new (void)
  * @...: a NULL ended list of #NcmData
  *
  * Creates a new #NcmDataset object and adds a NULL ended list
- * of #NcmData. 
+ * of #NcmData.
  *
  * Returns: a new #NcmDataset.
  */
@@ -203,17 +217,17 @@ ncm_dataset_new_list (gpointer data0, ...)
   if (data0 != NULL)
   {
     NcmData *data = NULL;
-    
+
     va_start (ap, data0);
 
     ncm_dataset_append_data (dset, data0);
-        
+
     while ((data = va_arg (ap, NcmData *)) != NULL)
       ncm_dataset_append_data (dset, data);
 
     va_end (ap);
   }
-  
+
   return dset;
 }
 
@@ -245,7 +259,8 @@ _ncm_dataset_update_bstrap (NcmDataset *dset)
     for (i = 0; i < dset->oa->len; i++)
     {
       NcmData *data = ncm_dataset_peek_data (dset, i);
-      gdouble p_i = ncm_data_get_length (data) * 1.0 / n;
+      gdouble p_i   = ncm_data_get_length (data) * 1.0 / n;
+
       g_array_index (dset->data_prob, gdouble, i) = p_i;
     }
   }
@@ -283,6 +298,7 @@ ncm_dataset_copy (NcmDataset *dset)
   for (i = 0; i < dset->oa->len; i++)
   {
     NcmData *data = ncm_dataset_peek_data (dset, i);
+
     ncm_obj_array_add (dset_dup->oa, G_OBJECT (data));
   }
 
@@ -330,6 +346,7 @@ ncm_dataset_get_n (NcmDataset *dset)
   for (i = 0; i < dset->oa->len; i++)
   {
     NcmData *data = ncm_dataset_peek_data (dset, i);
+
     n += ncm_data_get_length (data);
   }
 
@@ -354,13 +371,13 @@ ncm_dataset_get_dof (NcmDataset *dset)
   for (i = 0; i < dset->oa->len; i++)
   {
     NcmData *data = ncm_dataset_peek_data (dset, i);
+
     dof += ncm_data_get_dof (data);
   }
 
   return dof;
 }
 
-
 /**
  * ncm_dataset_all_init:
  * @dset: pointer to type defined by #NcmDataset
@@ -377,9 +394,11 @@ ncm_dataset_all_init (NcmDataset *dset)
   for (i = 0; i < dset->oa->len; i++)
   {
     NcmData *data = ncm_dataset_peek_data (dset, i);
-    if (!data->init)
+
+    if (!ncm_data_is_init (data))
       return FALSE;
   }
+
   return TRUE;
 }
 
@@ -425,6 +444,7 @@ NcmData *
 ncm_dataset_peek_data (NcmDataset *dset, guint n)
 {
   g_assert_cmpuint (n, <, dset->oa->len);
+
   return NCM_DATA (ncm_obj_array_peek (dset->oa, n));
 }
 
@@ -462,6 +482,7 @@ ncm_dataset_set_data_array (NcmDataset *dset, NcmObjArray *oa)
   for (i = 0; i < dset->oa->len; i++)
   {
     NcmData *data = ncm_dataset_peek_data (dset, i);
+
     if (dset->bstype == NCM_DATASET_BSTRAP_DISABLE)
       ncm_data_bootstrap_remove (data);
     else
@@ -540,6 +561,7 @@ ncm_dataset_resample (NcmDataset *dset, NcmMSet *mset, NcmRNG *rng)
   for (i = 0; i < dset->oa->len; i++)
   {
     NcmData *data = ncm_dataset_peek_data (dset, i);
+
     ncm_data_resample (data, mset, rng);
   }
 }
@@ -565,6 +587,7 @@ ncm_dataset_bootstrap_set (NcmDataset *dset, NcmDatasetBStrapType bstype)
     for (i = 0; i < dset->oa->len; i++)
     {
       NcmData *data = ncm_dataset_peek_data (dset, i);
+
       if (enable)
         ncm_data_bootstrap_create (data);
       else
@@ -588,35 +611,46 @@ void
 ncm_dataset_bootstrap_resample (NcmDataset *dset, NcmRNG *rng)
 {
   guint i;
+
   switch (dset->bstype)
   {
     case NCM_DATASET_BSTRAP_PARTIAL:
     {
       for (i = 0; i < dset->oa->len; i++)
       {
-        NcmData *data = ncm_dataset_peek_data (dset, i);
-        ncm_bootstrap_set_bsize (data->bstrap, data->bstrap->fsize);
+        NcmData *data        = ncm_dataset_peek_data (dset, i);
+        NcmBootstrap *bstrap = ncm_data_peek_bootstrap (data);
+        const guint fsize    = ncm_bootstrap_get_fsize (bstrap);
+
+        ncm_bootstrap_set_bsize (bstrap, fsize);
         ncm_data_bootstrap_resample (data, rng);
       }
+
       break;
     }
     case NCM_DATASET_BSTRAP_TOTAL:
     {
       guint n = ncm_dataset_get_n (dset);
+
       ncm_rng_lock (rng);
       gsl_ran_multinomial (rng->r, dset->oa->len, n,
-                           (gdouble *)dset->data_prob->data,
-                           (guint *)dset->bstrap->data);
+                           (gdouble *) dset->data_prob->data,
+                           (guint *) dset->bstrap->data);
       ncm_rng_unlock (rng);
 
       for (i = 0; i < dset->oa->len; i++)
       {
-        NcmData *data = ncm_dataset_peek_data (dset, i);
+        NcmData *data        = ncm_dataset_peek_data (dset, i);
+        NcmBootstrap *bstrap = ncm_data_peek_bootstrap (data);
+
         guint bsize = g_array_index (dset->bstrap, guint, i);
-        ncm_bootstrap_set_bsize (data->bstrap, bsize);
+
+        ncm_bootstrap_set_bsize (bstrap, bsize);
+
         if (bsize > 0)
           ncm_data_bootstrap_resample (data, rng);
       }
+
       break;
     }
     default:
@@ -639,10 +673,12 @@ ncm_dataset_log_info (NcmDataset *dset)
 
   ncm_cfg_msg_sepa ();
   g_message ("# Data used:\n");
+
   for (i = 0; i < dset->oa->len; i++)
   {
-    NcmData *data = ncm_dataset_peek_data (dset, i);
+    NcmData *data     = ncm_dataset_peek_data (dset, i);
     const gchar *desc = ncm_data_peek_desc (data);
+
     ncm_message_ww (desc,
                     "#   - ",
                     "#       ",
@@ -665,14 +701,16 @@ ncm_dataset_get_info (NcmDataset *dset)
 {
   guint i;
   GString *desc = g_string_new ("# Data used:\n");
+
   for (i = 0; i < dset->oa->len; i++)
   {
-    NcmData *data = ncm_dataset_peek_data (dset, i);
+    NcmData *data       = ncm_dataset_peek_data (dset, i);
     const gchar *desc_i = ncm_data_peek_desc (data);
-    gchar *desc_i_ww = ncm_string_ww (desc_i,
-                                      "#   - ",
-                                      "#       ",
-                                      80);
+    gchar *desc_i_ww    = ncm_string_ww (desc_i,
+                                         "#   - ",
+                                         "#       ",
+                                         80);
+
     g_string_append (desc, desc_i_ww);
     g_free (desc_i_ww);
   }
@@ -680,31 +718,34 @@ ncm_dataset_get_info (NcmDataset *dset)
   return g_string_free (desc, FALSE);
 }
 
-
-
 /**
  * ncm_dataset_has_leastsquares_f:
  * @dset: a #NcmDataset
  *
- * FIXME
+ * Whether all the #NcmData in @dset have a ncm_data_leastsquares_f() method.
  *
- * Returns: FIXME
+ * Returns: %TRUE if all the #NcmData in @dset have a ncm_data_leastsquares_f() method.
  */
 gboolean
 ncm_dataset_has_leastsquares_f (NcmDataset *dset)
 {
   if (dset->oa->len == 0)
+  {
     return FALSE;
+  }
   else
   {
     guint i;
+
     for (i = 0; i < dset->oa->len; i++)
     {
       NcmData *data = ncm_dataset_peek_data (dset, i);
+
       if (!NCM_DATA_GET_CLASS (data)->leastsquares_f)
         return FALSE;
     }
   }
+
   return TRUE;
 }
 
@@ -712,25 +753,30 @@ ncm_dataset_has_leastsquares_f (NcmDataset *dset)
  * ncm_dataset_has_leastsquares_J:
  * @dset: a #NcmDataset
  *
- * FIXME
+ * Whether all the #NcmData in @dset have a ncm_data_leastsquares_J() method.
  *
- * Returns: FIXME
+ * Returns: %TRUE if all the #NcmData in @dset have a ncm_data_leastsquares_J() method.
  */
 gboolean
 ncm_dataset_has_leastsquares_J (NcmDataset *dset)
 {
   if (dset->oa->len == 0)
+  {
     return FALSE;
+  }
   else
   {
     guint i;
+
     for (i = 0; i < dset->oa->len; i++)
     {
       NcmData *data = ncm_dataset_peek_data (dset, i);
+
       if (!NCM_DATA_GET_CLASS (data)->leastsquares_J)
         return FALSE;
     }
   }
+
   return TRUE;
 }
 
@@ -738,25 +784,30 @@ ncm_dataset_has_leastsquares_J (NcmDataset *dset)
  * ncm_dataset_has_leastsquares_f_J:
  * @dset: a #NcmDataset
  *
- * FIXME
+ * Whether all the #NcmData in @dset have a ncm_data_leastsquares_f_J() method.
  *
- * Returns: FIXME
+ * Returns: %TRUE if all the #NcmData in @dset have a ncm_data_leastsquares_f_J() method.
  */
 gboolean
 ncm_dataset_has_leastsquares_f_J (NcmDataset *dset)
 {
   if (dset->oa->len == 0)
+  {
     return FALSE;
+  }
   else
   {
     guint i;
+
     for (i = 0; i < dset->oa->len; i++)
     {
       NcmData *data = ncm_dataset_peek_data (dset, i);
+
       if (!NCM_DATA_GET_CLASS (data)->leastsquares_f_J)
         return FALSE;
     }
   }
+
   return TRUE;
 }
 
@@ -764,26 +815,30 @@ ncm_dataset_has_leastsquares_f_J (NcmDataset *dset)
  * ncm_dataset_has_m2lnL_val:
  * @dset: a #NcmDataset
  *
- * FIXME
- *
- * Returns: FIXME
+ * Whether all the #NcmData in @dset have a ncm_data_m2lnL_val() method.
  *
+ * Returns: %TRUE if all the #NcmData in @dset have a ncm_data_m2lnL_val() method.
  */
 gboolean
 ncm_dataset_has_m2lnL_val (NcmDataset *dset)
 {
   if (dset->oa->len == 0)
+  {
     return FALSE;
+  }
   else
   {
     guint i;
+
     for (i = 0; i < dset->oa->len; i++)
     {
       NcmData *data = ncm_dataset_peek_data (dset, i);
+
       if (!NCM_DATA_GET_CLASS (data)->m2lnL_val)
         return FALSE;
     }
   }
+
   return TRUE;
 }
 
@@ -791,26 +846,31 @@ ncm_dataset_has_m2lnL_val (NcmDataset *dset)
  * ncm_dataset_has_m2lnL_grad:
  * @dset: a #NcmDataset
  *
- * FIXME
+ * Whether all the #NcmData in @dset have a ncm_data_m2lnL_grad() method.
  *
- * Returns: FIXME
+ * Returns: %TRUE if all the #NcmData in @dset have a ncm_data_m2lnL_grad() method.
  *
  */
 gboolean
 ncm_dataset_has_m2lnL_grad (NcmDataset *dset)
 {
   if (dset->oa->len == 0)
+  {
     return FALSE;
+  }
   else
   {
     guint i;
+
     for (i = 0; i < dset->oa->len; i++)
     {
       NcmData *data = ncm_dataset_peek_data (dset, i);
+
       if (!NCM_DATA_GET_CLASS (data)->m2lnL_grad)
         return FALSE;
     }
   }
+
   return TRUE;
 }
 
@@ -818,26 +878,31 @@ ncm_dataset_has_m2lnL_grad (NcmDataset *dset)
  * ncm_dataset_has_m2lnL_val_grad:
  * @dset: a #NcmDataset
  *
- * FIXME
+ * Whether all the #NcmData in @dset have a ncm_data_m2lnL_val_grad() method.
  *
- * Returns: FIXME
+ * Returns: %TRUE if all the #NcmData in @dset have a ncm_data_m2lnL_val_grad() method.
  *
  */
 gboolean
 ncm_dataset_has_m2lnL_val_grad (NcmDataset *dset)
 {
   if (dset->oa->len == 0)
+  {
     return FALSE;
+  }
   else
   {
     guint i;
+
     for (i = 0; i < dset->oa->len; i++)
     {
       NcmData *data = ncm_dataset_peek_data (dset, i);
+
       if (!NCM_DATA_GET_CLASS (data)->m2lnL_val_grad)
         return FALSE;
     }
   }
+
   return TRUE;
 }
 
@@ -847,7 +912,10 @@ ncm_dataset_has_m2lnL_val_grad (NcmDataset *dset)
  * @mset: a #NcmMSet.
  * @f: a #NcmVector.
  *
- * FIXME
+ * Computes the leastsquares vector f for the data @data.
+ * The vector @f must be allocated with the correct size.
+ * The vector @f is filled with the values of the leastsquares vector f.
+ * 
  */
 void
 ncm_dataset_leastsquares_f (NcmDataset *dset, NcmMSet *mset, NcmVector *f)
@@ -857,13 +925,16 @@ ncm_dataset_leastsquares_f (NcmDataset *dset, NcmMSet *mset, NcmVector *f)
   for (i = 0; i < dset->oa->len; i++)
   {
     NcmData *data = ncm_dataset_peek_data (dset, i);
-    guint n = ncm_data_get_length (data);
+    guint n       = ncm_data_get_length (data);
 
     if (!NCM_DATA_GET_CLASS (data)->leastsquares_f)
+    {
       g_error ("ncm_dataset_leastsquares_f: %s dont implement leastsquares vector f", G_OBJECT_TYPE_NAME (data));
+    }
     else
     {
       NcmVector *f_i = ncm_vector_get_subvector (f, pos, n);
+
       ncm_data_prepare (data, mset);
       NCM_DATA_GET_CLASS (data)->leastsquares_f (data, mset, f_i);
       pos += n;
@@ -880,7 +951,9 @@ ncm_dataset_leastsquares_f (NcmDataset *dset, NcmMSet *mset, NcmVector *f)
  * @mset: a #NcmMSet.
  * @J: a #NcmMatrix.
  *
- * FIXME
+ * Computes the leastsquares matrix J for the data @data.
+ * The matrix @J must be allocated with the correct size.
+ * The matrix @J is filled with the values of the leastsquares matrix J.
  *
  */
 void
@@ -891,13 +964,16 @@ ncm_dataset_leastsquares_J (NcmDataset *dset, NcmMSet *mset, NcmMatrix *J)
   for (i = 0; i < dset->oa->len; i++)
   {
     NcmData *data = ncm_dataset_peek_data (dset, i);
-    guint n = ncm_data_get_length (data);
+    guint n       = ncm_data_get_length (data);
 
     if (!NCM_DATA_GET_CLASS (data)->leastsquares_J)
+    {
       g_error ("ncm_dataset_leastsquares_J: %s dont implement leastsquares matrix J", G_OBJECT_TYPE_NAME (data));
+    }
     else
     {
       NcmMatrix *J_i = ncm_matrix_get_submatrix (J, pos, 0, n, ncm_matrix_ncols (J));
+
       ncm_data_prepare (data, mset);
 
       NCM_DATA_GET_CLASS (data)->leastsquares_J (data, mset, J_i);
@@ -917,7 +993,12 @@ ncm_dataset_leastsquares_J (NcmDataset *dset, NcmMSet *mset, NcmMatrix *J)
  * @f: a #NcmVector.
  * @J: a #NcmMatrix.
  *
- * FIXME
+ * Computes the leastsquares vector f and matrix J for the data @data.
+ * The vector @f must be allocated with the correct size.
+ * The matrix @J must be allocated with the correct size.
+ * The vector @f is filled with the values of the leastsquares vector f.
+ * The matrix @J is filled with the values of the leastsquares matrix J.
+ * 
  */
 void
 ncm_dataset_leastsquares_f_J (NcmDataset *dset, NcmMSet *mset, NcmVector *f, NcmMatrix *J)
@@ -926,21 +1007,26 @@ ncm_dataset_leastsquares_f_J (NcmDataset *dset, NcmMSet *mset, NcmVector *f, Ncm
 
   for (i = 0; i < dset->oa->len; i++)
   {
-    NcmData *data = ncm_dataset_peek_data (dset, i);
-    guint n = ncm_data_get_length (data);
+    NcmData *data  = ncm_dataset_peek_data (dset, i);
+    guint n        = ncm_data_get_length (data);
     NcmMatrix *J_i = ncm_matrix_get_submatrix (J, pos, 0, n, ncm_matrix_ncols (J));
     NcmVector *f_i = ncm_vector_get_subvector (f, pos, n);
+
     ncm_data_prepare (data, mset);
 
     if (NCM_DATA_GET_CLASS (data)->leastsquares_f_J != NULL)
+    {
       NCM_DATA_GET_CLASS (data)->leastsquares_f_J (data, mset, f_i, J_i);
-    else if (NCM_DATA_GET_CLASS (data)->leastsquares_f != NULL && NCM_DATA_GET_CLASS (data)->leastsquares_J != NULL)
+    }
+    else if ((NCM_DATA_GET_CLASS (data)->leastsquares_f != NULL) && (NCM_DATA_GET_CLASS (data)->leastsquares_J != NULL))
     {
       NCM_DATA_GET_CLASS (data)->leastsquares_f (data, mset, f_i);
       NCM_DATA_GET_CLASS (data)->leastsquares_J (data, mset, J_i);
     }
     else
+    {
       g_error ("ncm_dataset_leastsquares_f_J: %s dont implement leastsquares f J", G_OBJECT_TYPE_NAME (data));
+    }
 
     pos += n;
     ncm_matrix_free (J_i);
@@ -952,14 +1038,17 @@ ncm_dataset_leastsquares_f_J (NcmDataset *dset, NcmMSet *mset, NcmVector *f, Ncm
  * ncm_dataset_m2lnL_val:
  * @dset: a #NcmLikelihood.
  * @mset: a #NcmMSet.
- * @m2lnL: (out): FIXME
+ * @m2lnL: (out): a pointer to a double.
  *
- * FIXME
+ * Computes the value of the m2lnL for the data @data.
+ * The value of the m2lnL is stored in @m2lnL.
+ * 
  */
 void
 ncm_dataset_m2lnL_val (NcmDataset *dset, NcmMSet *mset, gdouble *m2lnL)
 {
   guint i;
+
   *m2lnL = 0.0;
 
   for (i = 0; i < dset->oa->len; i++)
@@ -967,10 +1056,13 @@ ncm_dataset_m2lnL_val (NcmDataset *dset, NcmMSet *mset, gdouble *m2lnL)
     NcmData *data = ncm_dataset_peek_data (dset, i);
 
     if (!NCM_DATA_GET_CLASS (data)->m2lnL_val)
+    {
       g_error ("ncm_dataset_m2lnL_val: %s dont implement m2lnL", G_OBJECT_TYPE_NAME (data));
+    }
     else
     {
       gdouble m2lnL_i;
+
       ncm_data_prepare (data, mset);
       NCM_DATA_GET_CLASS (data)->m2lnL_val (data, mset, &m2lnL_i);
       *m2lnL += m2lnL_i;
@@ -986,8 +1078,9 @@ ncm_dataset_m2lnL_val (NcmDataset *dset, NcmMSet *mset, gdouble *m2lnL)
  * @mset: a #NcmMSet.
  * @m2lnL_v: a #NcmVector
  *
- * FIXME
- * 
+ * Computes the value of $-2\ln L$ for every element in the @dset.
+ * The values of $-2\ln L$ are stored in the #NcmVector @m2lnL_v.
+ *
  */
 void
 ncm_dataset_m2lnL_vec (NcmDataset *dset, NcmMSet *mset, NcmVector *m2lnL_v)
@@ -1001,10 +1094,13 @@ ncm_dataset_m2lnL_vec (NcmDataset *dset, NcmMSet *mset, NcmVector *m2lnL_v)
     NcmData *data = ncm_dataset_peek_data (dset, i);
 
     if (!NCM_DATA_GET_CLASS (data)->m2lnL_val)
+    {
       g_error ("ncm_dataset_m2lnL_val: %s dont implement m2lnL", G_OBJECT_TYPE_NAME (data));
+    }
     else
     {
       gdouble m2lnL_i;
+
       ncm_data_prepare (data, mset);
       NCM_DATA_GET_CLASS (data)->m2lnL_val (data, mset, &m2lnL_i);
       ncm_vector_set (m2lnL_v, i, m2lnL_i);
@@ -1020,7 +1116,9 @@ ncm_dataset_m2lnL_vec (NcmDataset *dset, NcmMSet *mset, NcmVector *m2lnL_v)
  * @mset: a #NcmMSet.
  * @grad: a #NcmVector.
  *
- * FIXME
+ * Computes the gradient of the m2lnL for the data @data.
+ * The gradient of the m2lnL is stored in @grad.
+ * 
  */
 void
 ncm_dataset_m2lnL_grad (NcmDataset *dset, NcmMSet *mset, NcmVector *grad)
@@ -1036,7 +1134,9 @@ ncm_dataset_m2lnL_grad (NcmDataset *dset, NcmMSet *mset, NcmVector *grad)
     NcmData *data = ncm_dataset_peek_data (dset, i);
 
     if (!NCM_DATA_GET_CLASS (data)->m2lnL_grad)
+    {
       g_error ("ncm_dataset_m2lnL_grad: %s dont implement m2lnL grad", G_OBJECT_TYPE_NAME (data));
+    }
     else
     {
       ncm_data_prepare (data, mset);
@@ -1054,10 +1154,13 @@ ncm_dataset_m2lnL_grad (NcmDataset *dset, NcmMSet *mset, NcmVector *grad)
  * ncm_dataset_m2lnL_val_grad:
  * @dset: a #NcmLikelihood.
  * @mset: a #NcmMSet.
- * @m2lnL: (out): FIXME
+ * @m2lnL: (out): a pointer to a double.
  * @grad: a #NcmVector.
  *
- * FIXME
+ * Computes the value of the m2lnL for the data @data.
+ * The value of the m2lnL is stored in @m2lnL.
+ * The gradient of the m2lnL is stored in @grad.
+ * The #NcmVector @grad must be previously allocated.
  */
 void
 ncm_dataset_m2lnL_val_grad (NcmDataset *dset, NcmMSet *mset, gdouble *m2lnL, NcmVector *grad)
@@ -1077,14 +1180,18 @@ ncm_dataset_m2lnL_val_grad (NcmDataset *dset, NcmMSet *mset, gdouble *m2lnL, Ncm
     ncm_data_prepare (data, mset);
 
     if (NCM_DATA_GET_CLASS (data)->m2lnL_val_grad != NULL)
+    {
       NCM_DATA_GET_CLASS (data)->m2lnL_val_grad (data, mset, &m2lnL_i, grad_i);
-    else if (NCM_DATA_GET_CLASS (data)->m2lnL_val != NULL && NCM_DATA_GET_CLASS (data)->m2lnL_grad != NULL)
+    }
+    else if ((NCM_DATA_GET_CLASS (data)->m2lnL_val != NULL) && (NCM_DATA_GET_CLASS (data)->m2lnL_grad != NULL))
     {
       NCM_DATA_GET_CLASS (data)->m2lnL_val (data, mset, &m2lnL_i);
       NCM_DATA_GET_CLASS (data)->m2lnL_grad (data, mset, grad_i);
     }
     else
+    {
       g_error ("ncm_dataset_m2lnL_val_grad: %s dont implement m2lnL val grad", G_OBJECT_TYPE_NAME (data));
+    }
 
     *m2lnL += m2lnL_i;
     ncm_vector_add (grad, grad_i);
@@ -1098,7 +1205,7 @@ ncm_dataset_m2lnL_val_grad (NcmDataset *dset, NcmMSet *mset, gdouble *m2lnL, Ncm
  * @dset: a #NcmLikelihood
  * @mset: a #NcmMSet
  * @i: an integer
- * @m2lnL_i: (out): FIXME
+ * @m2lnL_i: (out): a pointer to a double
  *
  * Get the value of the @i-th data in the dataset.
  *
@@ -1113,7 +1220,9 @@ ncm_dataset_m2lnL_i_val (NcmDataset *dset, NcmMSet *mset, guint i, gdouble *m2ln
     NcmData *data = ncm_dataset_peek_data (dset, i);
 
     if (!NCM_DATA_GET_CLASS (data)->m2lnL_val)
+    {
       g_error ("ncm_dataset_m2lnL_val: %s dont implement m2lnL", G_OBJECT_TYPE_NAME (data));
+    }
     else
     {
       ncm_data_prepare (data, mset);
@@ -1132,9 +1241,11 @@ ncm_dataset_m2lnL_i_val (NcmDataset *dset, NcmMSet *mset, guint i, gdouble *m2ln
  *
  * Calculates the Fisher-information matrix @I adding
  * the individual ones from each #NcmData in @dset.
- * 
+ * If the #NcmMatrix pointer in *@IM is NULL a new #NcmMatrix 
+ * will be allocated otherwise *@IM will be used.
+ *
  */
-void 
+void
 ncm_dataset_fisher_matrix (NcmDataset *dset, NcmMSet *mset, NcmMatrix **IM)
 {
   const guint fparams_len = ncm_mset_fparams_len (mset);
@@ -1148,8 +1259,10 @@ ncm_dataset_fisher_matrix (NcmDataset *dset, NcmMSet *mset, NcmMatrix **IM)
   for (i = 0; i < dset->oa->len; i++)
   {
     NcmData *data = ncm_dataset_peek_data (dset, i);
+
     ncm_data_fisher_matrix (data, mset, &IM0);
 
     ncm_matrix_add (*IM, IM0);
   }
 }
+
