@@ -54,7 +54,7 @@ struct _NcmStatsDistClass
 {
   /*< private >*/
   GObjectClass parent_class;
-  
+
   void (*set_dim) (NcmStatsDist *sd, const guint dim);
   gdouble (*get_href) (NcmStatsDist *sd);
   void (*prepare_kernel) (NcmStatsDist *sd, GPtrArray *sample_array);
@@ -79,6 +79,7 @@ struct _NcmStatsDist
  * NcmStatsDistCV:
  * @NCM_STATS_DIST_CV_NONE: No cross validation
  * @NCM_STATS_DIST_CV_SPLIT: Sample split cross validation
+ * @NCM_STATS_DIST_CV_SPLIT_NOFIT: Sample split cross validation without fitting
  *
  * Cross-validation method to be applied.
  *
@@ -87,6 +88,7 @@ typedef enum _NcmStatsDistCV
 {
   NCM_STATS_DIST_CV_NONE,
   NCM_STATS_DIST_CV_SPLIT,
+  NCM_STATS_DIST_CV_SPLIT_NOFIT,
   /* < private > */
   NCM_STATS_DIST_CV_LEN, /*< skip >*/
 } NcmStatsDistCV;
@@ -103,6 +105,7 @@ NcmStatsDistKernel *ncm_stats_dist_get_kernel (NcmStatsDist *sd);
 
 guint ncm_stats_dist_get_dim (NcmStatsDist *sd);
 guint ncm_stats_dist_get_sample_size (NcmStatsDist *sd);
+guint ncm_stats_dist_get_n_kernels (NcmStatsDist *sd);
 gdouble ncm_stats_dist_get_href (NcmStatsDist *sd);
 
 void ncm_stats_dist_set_over_smooth (NcmStatsDist *sd, const gdouble over_smooth);
