@@ -72,12 +72,12 @@
  * The #NcmStatsDist class implements the solution for this relation and then one can compute the distribution for a given
  * vector $\vec{x}$ using a method of the #NcmStatsDist class but that is implemented in this object.
  *
- * 
- * The user must provide input the values: @sdk, @CV_type - ncm_stats_dist_kde_new(), @y - ncm_stats_dist_add_obs(), @split_frac - ncm_stats_dist_set_split_frac(), 
+ *
+ * The user must provide input the values: @sdk, @CV_type - ncm_stats_dist_kde_new(), @y - ncm_stats_dist_add_obs(), @split_frac - ncm_stats_dist_set_split_frac(),
  * @over_smooth - ncm_stats_dist_set_over_smooth(), $v(x)$ - ncm_stats_dist_prepare_interp().
  * To see an example of how to use this object and the main functions that are called within each function,
  * check the fluxogram at the end of this documentation,
- * where the order of the functions that should be called by the user and some of the functions that the algorithm calls. 
+ * where the order of the functions that should be called by the user and some of the functions that the algorithm calls.
  *
  * ![kde_sketch](kde.png)
  *
@@ -118,28 +118,29 @@ static void
 ncm_stats_dist_kde_init (NcmStatsDistKDE *sdkde)
 {
   NcmStatsDistKDEPrivate * const self = sdkde->priv = ncm_stats_dist_kde_get_instance_private (sdkde);
-  
-  self->sample         = NULL;
-  self->cov_type       = NCM_STATS_DIST_KDE_COV_TYPE_LEN;
-  self->cov_fixed      = NULL;
-  self->cov_decomp     = NULL;
-  self->sample_matrix  = NULL;
-  self->invUsample     = g_ptr_array_new ();
-  self->v              = NULL;
-  self->chi2           = NULL;
-  self->kernel_lnnorm  = 0.0;
-  self->nearPD_maxiter = 0;
-  
-  g_ptr_array_set_free_func (self->invUsample, (GDestroyNotify) ncm_vector_free);
+
+  self->sample            = NULL;
+  self->cov_type          = NCM_STATS_DIST_KDE_COV_TYPE_LEN;
+  self->cov_fixed         = NULL;
+  self->cov_decomp        = NULL;
+  self->sample_matrix     = NULL;
+  self->invUsample_matrix = NULL;
+  self->invUsample_array  = g_ptr_array_new ();
+  self->v                 = NULL;
+  self->chi2              = NULL;
+  self->kernel_lnnorm     = 0.0;
+  self->nearPD_maxiter    = 0;
+
+  g_ptr_array_set_free_func (self->invUsample_array, (GDestroyNotify) ncm_vector_free);
 }
 
 static void
 _ncm_stats_dist_kde_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
   NcmStatsDistKDE *sdkde = NCM_STATS_DIST_KDE (object);
-  
+
   /*g_return_if_fail (NCM_IS_STATS_DIST_KDE (object));*/
-  
+
   switch (prop_id)
   {
     case PROP_NEARPD_MAXITER:
@@ -160,11 +161,12 @@ _ncm_stats_dist_kde_set_property (GObject *object, guint prop_id, const GValue *
 static void
 _ncm_stats_dist_kde_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
-  NcmStatsDistKDE *sdkde              = NCM_STATS_DIST_KDE (object);
+  NcmStatsDistKDE *sdkde = NCM_STATS_DIST_KDE (object);
+
   /*NcmStatsDistKDEPrivate * const self = sdkde->priv;*/
-  
+
   g_return_if_fail (NCM_IS_STATS_DIST_KDE (object));
-  
+
   switch (prop_id)
   {
     case PROP_NEARPD_MAXITER:
@@ -187,16 +189,17 @@ _ncm_stats_dist_kde_dispose (GObject *object)
 {
   NcmStatsDistKDE *sdkde              = NCM_STATS_DIST_KDE (object);
   NcmStatsDistKDEPrivate * const self = sdkde->priv;
-  
+
   ncm_stats_vec_clear (&self->sample);
   ncm_matrix_clear (&self->cov_fixed);
   ncm_matrix_clear (&self->cov_decomp);
   ncm_matrix_clear (&self->sample_matrix);
+  ncm_matrix_clear (&self->invUsample_matrix);
   ncm_vector_clear (&self->v);
   ncm_vector_clear (&self->chi2);
 
-  g_clear_pointer (&self->invUsample, g_ptr_array_unref);
-  
+  g_clear_pointer (&self->invUsample_array, g_ptr_array_unref);
+
   /* Chain up : end */
   G_OBJECT_CLASS (ncm_stats_dist_kde_parent_class)->dispose (object);
 }
@@ -206,7 +209,7 @@ _ncm_stats_dist_kde_finalize (GObject *object)
 {
   /*NcmStatsDistKDE *sdkde = NCM_STATS_DIST_KDE (object);*/
   /*NcmStatsDistKDEPrivate * const self = sdkde->priv;*/
-  
+
   /* Chain up : end */
   G_OBJECT_CLASS (ncm_stats_dist_kde_parent_class)->finalize (object);
 }
@@ -224,12 +227,12 @@ ncm_stats_dist_kde_class_init (NcmStatsDistKDEClass *klass)
 {
   GObjectClass *object_class  = G_OBJECT_CLASS (klass);
   NcmStatsDistClass *sd_class = NCM_STATS_DIST_CLASS (klass);
-  
+
   object_class->set_property = &_ncm_stats_dist_kde_set_property;
   object_class->get_property = &_ncm_stats_dist_kde_get_property;
   object_class->dispose      = &_ncm_stats_dist_kde_dispose;
   object_class->finalize     = &_ncm_stats_dist_kde_finalize;
-  
+
   g_object_class_install_property (object_class,
                                    PROP_NEARPD_MAXITER,
                                    g_param_spec_uint ("nearPD-maxiter",
@@ -252,7 +255,7 @@ ncm_stats_dist_kde_class_init (NcmStatsDistKDEClass *klass)
                                                         NCM_TYPE_MATRIX,
                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
 
-  
+
   sd_class->set_dim            = &_ncm_stats_dist_kde_set_dim;
   sd_class->prepare_kernel     = &_ncm_stats_dist_kde_prepare_kernel;
   sd_class->compute_IM         = &_ncm_stats_dist_kde_compute_IM;
@@ -270,13 +273,14 @@ _ncm_stats_dist_kde_set_dim (NcmStatsDist *sd, const guint dim)
   {
     NcmStatsDistKDE *sdkde              = NCM_STATS_DIST_KDE (sd);
     NcmStatsDistKDEPrivate * const self = sdkde->priv;
-    
+
     ncm_stats_vec_clear (&self->sample);
-    
+
     ncm_matrix_clear (&self->cov_decomp);
     ncm_matrix_clear (&self->sample_matrix);
+    ncm_matrix_clear (&self->invUsample_matrix);
     ncm_vector_clear (&self->v);
-    
+
     self->sample     = ncm_stats_vec_new (dim, NCM_STATS_VEC_COV, TRUE);
     self->cov_decomp = ncm_matrix_new (dim, dim);
     self->v          = ncm_vector_new (dim);
@@ -287,22 +291,22 @@ static void
 _cholesky_decomp (NcmMatrix *cov_decomp, NcmMatrix *cov, const guint d, const guint maxiter)
 {
   ncm_matrix_memcpy (cov_decomp, cov);
-  
+
   if (ncm_matrix_cholesky_decomp (cov_decomp, 'U') != 0)
   {
     ncm_matrix_memcpy (cov_decomp, cov);
-    
+
     if (ncm_matrix_nearPD (cov_decomp, 'U', TRUE, maxiter) != 0)
     {
       gint i;
-      
+
       ncm_matrix_set_zero (cov_decomp);
-      
+
       for (i = 0; i < d; i++)
       {
         ncm_matrix_set (cov_decomp, i, i, ncm_matrix_get (cov, i, i));
       }
-      
+
       g_assert_cmpint (ncm_matrix_cholesky_decomp (cov_decomp, 'U'), ==, 0);
     }
   }
@@ -315,19 +319,19 @@ _ncm_stats_dist_kde_prepare_kernel (NcmStatsDist *sd, GPtrArray *sample_array)
   NcmStatsDistKDEPrivate * const self = sdkde->priv;
   NcmStatsDistPrivate * const pself = sd->priv;
   gint i, ret;
-  
+
   /*
    * Computing the covariance matrix considering the whole sample.
    */
   ncm_stats_vec_reset (self->sample, TRUE);
-  
-  for (i = 0; i < pself->n; i++)
+
+  for (i = 0; i < pself->n_kernels; i++)
   {
     NcmVector *theta_i = g_ptr_array_index (sample_array, i);
-    
+
     ncm_stats_vec_append (self->sample, theta_i, FALSE);
   }
-  
+
   /*
    * Computing the Cholesky decomposition of the total covariance.
    */
@@ -338,6 +342,7 @@ _ncm_stats_dist_kde_prepare_kernel (NcmStatsDist *sd, GPtrArray *sample_array)
     case NCM_STATS_DIST_KDE_COV_TYPE_SAMPLE:
     {
       NcmMatrix *cov = ncm_stats_vec_peek_cov_matrix (self->sample, 0);
+
       _cholesky_decomp (self->cov_decomp, cov, pself->d, self->nearPD_maxiter);
       break;
     }
@@ -351,6 +356,7 @@ _ncm_stats_dist_kde_prepare_kernel (NcmStatsDist *sd, GPtrArray *sample_array)
     case NCM_STATS_DIST_KDE_COV_TYPE_ROBUST_DIAG:
     {
       NcmMatrix *cov = ncm_stats_vec_compute_cov_robust_diag (self->sample);
+
       _cholesky_decomp (self->cov_decomp, cov, pself->d, self->nearPD_maxiter);
       ncm_matrix_free (cov);
 
@@ -359,6 +365,7 @@ _ncm_stats_dist_kde_prepare_kernel (NcmStatsDist *sd, GPtrArray *sample_array)
     case NCM_STATS_DIST_KDE_COV_TYPE_ROBUST:
     {
       NcmMatrix *cov = ncm_stats_vec_compute_cov_robust_ogk (self->sample);
+
       _cholesky_decomp (self->cov_decomp, cov, pself->d, self->nearPD_maxiter);
       ncm_matrix_free (cov);
     }
@@ -367,40 +374,49 @@ _ncm_stats_dist_kde_prepare_kernel (NcmStatsDist *sd, GPtrArray *sample_array)
       g_assert_not_reached ();
       break;
   }
-  
+
   /*
    * Getting kernel normalization
    */
   self->kernel_lnnorm = ncm_stats_dist_kernel_get_lnnorm (pself->kernel, self->cov_decomp);
-  
-  if ((self->sample_matrix == NULL) || (pself->n != ncm_matrix_nrows (self->sample_matrix)) || (pself->d != ncm_matrix_ncols (self->sample_matrix)))
+
+  if ((self->sample_matrix == NULL) ||
+      (pself->n_obs != ncm_matrix_nrows (self->sample_matrix)) ||
+      (pself->d != ncm_matrix_ncols (self->sample_matrix)))
   {
     ncm_matrix_clear (&self->sample_matrix);
-    self->sample_matrix = ncm_matrix_new (pself->n, pself->d);
-    
-    g_ptr_array_set_size (self->invUsample, 0);
-    
-    for (i = 0; i < pself->n; i++)
+    ncm_matrix_clear (&self->invUsample_matrix);
+
+    self->sample_matrix     = ncm_matrix_new (pself->n_obs, pself->d);
+    self->invUsample_matrix = ncm_matrix_new (pself->n_obs, pself->d);
+
+    g_ptr_array_set_size (self->invUsample_array, 0);
+
+    for (i = 0; i < pself->n_obs; i++)
     {
-      NcmVector *row_i = ncm_matrix_get_row (self->sample_matrix, i);
-      
-      g_ptr_array_add (self->invUsample, row_i);
+      NcmVector *row_i = ncm_matrix_get_row (self->invUsample_matrix, i);
+
+      g_ptr_array_add (self->invUsample_array, row_i);
     }
   }
-  
-  for (i = 0; i < pself->n; i++)
+
+  for (i = 0; i < pself->n_obs; i++)
     ncm_matrix_set_row (self->sample_matrix, i, g_ptr_array_index (sample_array, i));
-  
-  ret = gsl_blas_dtrsm (CblasRight, CblasUpper, CblasNoTrans, CblasNonUnit, 1.0, ncm_matrix_gsl (self->cov_decomp), ncm_matrix_gsl (self->sample_matrix));
+
+  ncm_matrix_memcpy (self->invUsample_matrix, self->sample_matrix);
+
+  ret = gsl_blas_dtrsm (CblasRight, CblasUpper, CblasNoTrans, CblasNonUnit,
+                        1.0, ncm_matrix_gsl (self->cov_decomp),
+                        ncm_matrix_gsl (self->invUsample_matrix));
   NCM_TEST_GSL_RESULT ("_ncm_stats_dist_kde_prepare_kernel", ret);
-  
+
   /*
    * Allocating the evaluation vector
    */
-  if ((self->chi2 == NULL) || (ncm_vector_len (self->chi2) != pself->n))
+  if ((self->chi2 == NULL) || (ncm_vector_len (self->chi2) != pself->n_kernels))
   {
     ncm_vector_clear (&self->chi2);
-    self->chi2 = ncm_vector_new (pself->n);
+    self->chi2 = ncm_vector_new (pself->n_kernels);
   }
 }
 
@@ -412,35 +428,58 @@ _ncm_stats_dist_kde_compute_IM (NcmStatsDist *sd, NcmMatrix *IM)
   NcmStatsDistPrivate * const pself   = sd->priv;
   const gdouble href2                 = pself->href * pself->href;
   gint i;
-  
-  for (i = 0; i < pself->n; i++)
+
+  for (i = 0; i < pself->n_kernels; i++)
   {
-    NcmVector *row_i = g_ptr_array_index (self->invUsample, i);
+    NcmVector *row_i = g_ptr_array_index (self->invUsample_array, i);
     gint j;
-    
+
     ncm_matrix_set (IM, i, i, 0.0);
-    
-    for (j = i + 1; j < pself->n; j++)
+
+    for (j = i + 1; j < pself->n_kernels; j++)
     {
-      NcmVector *row_j = g_ptr_array_index (self->invUsample, j);
+      NcmVector *row_j = g_ptr_array_index (self->invUsample_array, j);
       gdouble chi2_ij  = 0.0;
       gint k;
-      
+
       for (k = 0; k < pself->d; k++)
       {
         chi2_ij += gsl_pow_2 ((ncm_vector_fast_get (row_i, k) - ncm_vector_fast_get (row_j, k)));
       }
-      
+
       chi2_ij = chi2_ij / href2;
-      
+
       ncm_matrix_set (IM, i, j, chi2_ij);
       ncm_matrix_set (IM, j, i, chi2_ij);
     }
   }
 
-  for (i = 0; i < pself->n; i++)
+  for (i = pself->n_kernels; i < pself->n_obs; i++)
+  {
+    NcmVector *row_i = g_ptr_array_index (self->invUsample_array, i);
+    gint j;
+
+    for (j = 0; j < pself->n_kernels; j++)
+    {
+      NcmVector *row_j = g_ptr_array_index (self->invUsample_array, j);
+      gdouble chi2_ij  = 0.0;
+      gint k;
+
+      for (k = 0; k < pself->d; k++)
+      {
+        chi2_ij += gsl_pow_2 ((ncm_vector_fast_get (row_i, k) - ncm_vector_fast_get (row_j, k)));
+      }
+
+      chi2_ij = chi2_ij / href2;
+
+      ncm_matrix_set (IM, i, j, chi2_ij);
+    }
+  }
+
+  for (i = 0; i < pself->n_obs; i++)
   {
     NcmVector *row_i = ncm_matrix_get_row (IM, i);
+
     ncm_stats_dist_kernel_eval_unnorm_vec (pself->kernel, row_i, row_i);
     ncm_vector_free (row_i);
   }
@@ -453,7 +492,7 @@ _ncm_stats_dist_kde_peek_cov_decomp (NcmStatsDist *sd, guint i)
 {
   NcmStatsDistKDE *sdkde              = NCM_STATS_DIST_KDE (sd);
   NcmStatsDistKDEPrivate * const self = sdkde->priv;
-  
+
   return self->cov_decomp;
 }
 
@@ -463,7 +502,7 @@ _ncm_stats_dist_kde_get_lnnorm (NcmStatsDist *sd, guint i)
   NcmStatsDistKDE *sdkde              = NCM_STATS_DIST_KDE (sd);
   NcmStatsDistKDEPrivate * const self = sdkde->priv;
   NcmStatsDistPrivate * const pself   = sd->priv;
-  
+
   return self->kernel_lnnorm + pself->d * log (pself->href);
 }
 
@@ -475,30 +514,30 @@ _ncm_stats_dist_kde_eval_weights (NcmStatsDist *sd, NcmVector *weights, NcmVecto
   NcmStatsDistPrivate * const pself = sd->priv;
   const gdouble href2 = pself->href * pself->href;
   gint i, ret;
-  
+
   ncm_vector_memcpy (self->v, x);
   ret = gsl_blas_dtrsv (CblasUpper, CblasTrans, CblasNonUnit,
                         ncm_matrix_gsl (self->cov_decomp), ncm_vector_gsl (self->v));
   NCM_TEST_GSL_RESULT ("ncm_stats_dist_nd_eval", ret);
-  
-  for (i = 0; i < pself->n; i++)
+
+  for (i = 0; i < pself->n_kernels; i++)
   {
-    NcmVector *row_i = g_ptr_array_index (self->invUsample, i);
+    NcmVector *row_i = g_ptr_array_index (self->invUsample_array, i);
     gdouble chi2_i   = 0.0;
     gint k;
-    
+
     for (k = 0; k < pself->d; k++)
     {
       chi2_i += gsl_pow_2 ((ncm_vector_fast_get (row_i, k) - ncm_vector_fast_get (self->v, k)));
     }
-    
+
     chi2_i = chi2_i / href2;
-    
+
     ncm_vector_fast_set (self->chi2, i, chi2_i);
   }
-  
+
   ncm_stats_dist_kernel_eval_unnorm_vec (pself->kernel, self->chi2, self->chi2);
-  
+
   return ncm_vector_dot (self->chi2, pself->weights) * exp (-(self->kernel_lnnorm + pself->d * log (pself->href)));
 }
 
@@ -510,33 +549,33 @@ _ncm_stats_dist_kde_eval_weights_m2lnp (NcmStatsDist *sd, NcmVector *weights, Nc
   NcmStatsDistPrivate * const pself = sd->priv;
   const gdouble href2 = pself->href * pself->href;
   gint i, ret;
-  
+
   ncm_vector_memcpy (self->v, x);
   ret = gsl_blas_dtrsv (CblasUpper, CblasTrans, CblasNonUnit,
                         ncm_matrix_gsl (self->cov_decomp), ncm_vector_gsl (self->v));
   NCM_TEST_GSL_RESULT ("ncm_stats_dist_nd_eval", ret);
-  
-  for (i = 0; i < pself->n; i++)
+
+  for (i = 0; i < pself->n_kernels; i++)
   {
-    NcmVector *row_i = g_ptr_array_index (self->invUsample, i);
+    NcmVector *row_i = g_ptr_array_index (self->invUsample_array, i);
     gdouble chi2_i   = 0.0;
     gint k;
-    
+
     for (k = 0; k < pself->d; k++)
     {
       chi2_i += gsl_pow_2 ((ncm_vector_fast_get (row_i, k) - ncm_vector_fast_get (self->v, k)));
     }
-    
+
     chi2_i = chi2_i / href2;
-    
+
     ncm_vector_fast_set (self->chi2, i, chi2_i);
   }
-  
+
   {
     gdouble gamma, lambda;
-    
+
     ncm_stats_dist_kernel_eval_sum1_gamma_lambda (pself->kernel, self->chi2, pself->weights, self->kernel_lnnorm, &gamma, &lambda);
-    
+
     return -2.0 * (gamma + log1p (lambda) - pself->d * log (pself->href));
   }
 }
@@ -558,7 +597,7 @@ ncm_stats_dist_kde_new (NcmStatsDistKernel *sdk, NcmStatsDistCV CV_type)
                                          "kernel", sdk,
                                          "CV-type", CV_type,
                                          NULL);
-  
+
   return sdkde;
 }
 
@@ -609,14 +648,14 @@ ncm_stats_dist_kde_clear (NcmStatsDistKDE **sdkde)
  *
  * Sets the maximum number of iterations when finding the
  * nearest positive definite covariance matrix to @maxiter. This function is implemented
- * as a property and is called in the _cholesky_decomp and in the @ncm_stats_dist_kde_prepare_kernel function. 
+ * as a property and is called in the _cholesky_decomp and in the @ncm_stats_dist_kde_prepare_kernel function.
  *
  */
 void
 ncm_stats_dist_kde_set_nearPD_maxiter (NcmStatsDistKDE *sdkde, const guint maxiter)
 {
   NcmStatsDistKDEPrivate * const self = sdkde->priv;
-  
+
   self->nearPD_maxiter = maxiter;
 }
 
@@ -630,7 +669,7 @@ guint
 ncm_stats_dist_kde_get_nearPD_maxiter (NcmStatsDistKDE *sdkde)
 {
   NcmStatsDistKDEPrivate * const self = sdkde->priv;
-  
+
   return self->nearPD_maxiter;
 }
 
@@ -648,13 +687,13 @@ ncm_stats_dist_kde_set_cov_type (NcmStatsDistKDE *sdkde, NcmStatsDistKDECovType
   NcmStatsDistKDEPrivate * const self = sdkde->priv;
 
   self->cov_type = cov_type;
+
   if ((self->cov_type == NCM_STATS_DIST_KDE_COV_TYPE_FIXED) && (self->cov_fixed != NULL))
   {
     ncm_matrix_memcpy (self->cov_decomp, self->cov_fixed);
+
     if (ncm_matrix_cholesky_decomp (self->cov_decomp, 'U') != 0)
-    {
       g_error ("ncm_stats_dist_kde_set_cov_fixed: matrix cov_fixed is not positive definite.");
-    }
   }
 }
 
@@ -686,7 +725,7 @@ void
 ncm_stats_dist_kde_set_cov_fixed (NcmStatsDistKDE *sdkde, NcmMatrix *cov_fixed)
 {
   NcmStatsDistKDEPrivate * const self = sdkde->priv;
-  NcmStatsDistPrivate * const pself = NCM_STATS_DIST (sdkde)->priv;
+  NcmStatsDistPrivate * const pself   = NCM_STATS_DIST (sdkde)->priv;
 
   g_assert_cmpuint (ncm_matrix_ncols (cov_fixed), ==, pself->d);
   g_assert_cmpuint (ncm_matrix_nrows (cov_fixed), ==, pself->d);
@@ -698,10 +737,9 @@ ncm_stats_dist_kde_set_cov_fixed (NcmStatsDistKDE *sdkde, NcmMatrix *cov_fixed)
   if (self->cov_type == NCM_STATS_DIST_KDE_COV_TYPE_FIXED)
   {
     ncm_matrix_memcpy (self->cov_decomp, self->cov_fixed);
+
     if (ncm_matrix_cholesky_decomp (self->cov_decomp, 'U') != 0)
-    {
       g_error ("ncm_stats_dist_kde_set_cov_fixed: matrix cov_fixed is not positive definite.");
-    }
   }
 }
 
@@ -720,3 +758,4 @@ ncm_stats_dist_kde_peek_cov_fixed (NcmStatsDistKDE *sdkde)
 
   return self->cov_fixed;
 }
+
