@@ -32,6 +32,10 @@
 #include "math/ncm_stats_dist.h"
 #include "math/ncm_nnls.h"
 
+#ifndef NUMCOSMO_GIR_SCAN
+#include <gsl/gsl_multimin.h>
+#endif /* NUMCOSMO_GIR_SCAN */
+
 G_BEGIN_DECLS
 
 struct _NcmStatsDistPrivate
@@ -50,8 +54,10 @@ struct _NcmStatsDistPrivate
   gdouble max_m2lnp;
   gdouble href;
   gdouble rnorm;
-  guint n;
-  guint alloc_n;
+  guint n_obs;
+  guint n_kernels;
+  guint alloc_n_obs;
+  guint alloc_n_kernels;
   gboolean alloc_subs;
   guint d;
   GArray *sampling;
@@ -63,7 +69,9 @@ struct _NcmStatsDistPrivate
   NcmVector *f1;
   gdouble *levmar_workz;
   guint levmar_n;
+  gsl_multimin_fminimizer *fmin;
   GArray *m2lnp_sort;
+  NcmRNG *rng;
 };
 
 G_END_DECLS
