@@ -70,8 +70,8 @@ static void test_ncm_stats_dist_kernel_invalid_stub (TestNcmStatsDistKernel *tes
 typedef struct _TestNcmStatsDistKernelFunc
 {
   const gchar *name;
-  
-  void (*test_func)(TestNcmStatsDistKernel *test, gconstpointer pdata);
+
+  void (*test_func) (TestNcmStatsDistKernel *test, gconstpointer pdata);
 } TestNcmStatsDistKernelFunc;
 
 #define TEST_NCM_STATS_DIST_KERNEL_CONSTRUCTORS_LEN 2
@@ -94,38 +94,38 @@ gint
 main (gint argc, gchar *argv[])
 {
   gint i, j;
-  
+
   g_test_init (&argc, &argv, NULL);
-  
+
   ncm_cfg_init_full_ptr (&argc, &argv);
   ncm_cfg_enable_gsl_err_handler ();
-  
+
   g_test_set_nonfatal_assertions ();
-  
+
   for (i = 0; i < TEST_NCM_STATS_DIST_KERNEL_CONSTRUCTORS_LEN; i++)
   {
     for (j = 0; j < TEST_NCM_STATS_DIST_KERNEL_TESTS_LEN; j++)
     {
       gchar *test_name = g_strdup_printf ("/ncm/stats/dist/kernel/%s/%s", constructors[i].name, tests[j].name);
-      
+
       g_test_add (test_name, TestNcmStatsDistKernel, NULL, constructors[i].test_func, tests[j].test_func, &test_ncm_stats_dist_kernel_free);
       g_free (test_name);
     }
   }
-  
+
   g_test_add ("/ncm/stats/dist/kernel/gauss/traps", TestNcmStatsDistKernel, NULL,
               &test_ncm_stats_dist_kernel_new_gauss,
               &test_ncm_stats_dist_kernel_traps,
               &test_ncm_stats_dist_kernel_free);
-  
+
 #if GLIB_CHECK_VERSION (2, 38, 0)
   g_test_add ("/ncm/stats/dist/kernel/gauss/invalid/stub/subprocess", TestNcmStatsDistKernel, NULL,
               &test_ncm_stats_dist_kernel_new_gauss,
               &test_ncm_stats_dist_kernel_invalid_stub,
               &test_ncm_stats_dist_kernel_free);
 #endif
-  
-  
+
+
   g_test_run ();
 }
 
@@ -134,17 +134,17 @@ test_ncm_stats_dist_kernel_new_gauss (TestNcmStatsDistKernel *test, gconstpointe
 {
   const guint dim                    = g_test_rand_int_range (2, 4);
   NcmStatsDistKernelGauss *sdk_gauss = ncm_stats_dist_kernel_gauss_new (dim);
-  
+
   test->dim         = dim;
   test->kernel      = NCM_STATS_DIST_KERNEL (sdk_gauss);
   test->kernel_type = NCM_STATS_DIST_KERNEL_GAUSS;
   test->nfail       = 0;
-  
+
   ncm_stats_dist_kernel_gauss_ref (sdk_gauss);
   ncm_stats_dist_kernel_gauss_free (sdk_gauss);
   {
     NcmStatsDistKernelGauss *sdk_gauss0 = ncm_stats_dist_kernel_gauss_ref (sdk_gauss);
-    
+
     ncm_stats_dist_kernel_gauss_clear (&sdk_gauss0);
     g_assert_true (sdk_gauss0 == NULL);
   }
@@ -156,18 +156,18 @@ test_ncm_stats_dist_kernel_new_st (TestNcmStatsDistKernel *test, gconstpointer p
   const gdouble nu             = g_test_rand_double_range (3.0, 5.0);
   const guint dim              = g_test_rand_int_range (2, 4);
   NcmStatsDistKernelST *sdk_st = ncm_stats_dist_kernel_st_new (dim, nu);
-  
+
   test->dim         = dim;
   test->nu          = nu;
   test->kernel_type = NCM_STATS_DIST_KERNEL_ST3;
   test->kernel      = NCM_STATS_DIST_KERNEL (sdk_st);
   test->nfail       = 0;
-  
+
   ncm_stats_dist_kernel_st_ref (sdk_st);
   ncm_stats_dist_kernel_st_free (sdk_st);
   {
     NcmStatsDistKernelST *sdk_st0 = ncm_stats_dist_kernel_st_ref (sdk_st);
-    
+
     ncm_stats_dist_kernel_st_clear (&sdk_st0);
     g_assert_true (sdk_st0 == NULL);
   }
@@ -185,7 +185,7 @@ test_ncm_stats_dist_kernel_bandwidth (TestNcmStatsDistKernel *test, gconstpointe
   const gdouble n = g_test_rand_double_range (1.0, 200.0);
   const gdouble h = ncm_stats_dist_kernel_get_rot_bandwidth (test->kernel, n);
   gdouble h_test  = 0.0;
-  
+
   switch (test->kernel_type)
   {
     case NCM_STATS_DIST_KERNEL_GAUSS:
@@ -198,7 +198,7 @@ test_ncm_stats_dist_kernel_bandwidth (TestNcmStatsDistKernel *test, gconstpointe
       g_assert_not_reached ();
       break;
   }
-  
+
   ncm_assert_cmpdouble_e (h_test, ==, h, 1.0e-14, 0.0);
 }
 
@@ -217,29 +217,29 @@ test_ncm_stats_dist_kernel_norm (TestNcmStatsDistKernel *test, gconstpointer pda
   guint i;
   NcmMatrix *cov    = NULL;
   gdouble lndet_cov = 0.0;
-  
+
   ncm_data_gauss_cov_use_norma (NCM_DATA_GAUSS_COV (data_mvnd), FALSE);
   ncm_mset_param_set_vector (mset, ncm_data_gauss_cov_mvnd_peek_mean (data_mvnd));
-  
+
   for (i = 0; i < np; i++)
   {
     gdouble m2lnL;
-    
+
     ncm_data_m2lnL_val (NCM_DATA (data_mvnd), mset, &m2lnL);
     ncm_vector_set (m2lnp_v, i, m2lnL);
   }
-  
-  cov       = NCM_DATA_GAUSS_COV (data_mvnd)->cov;
-  lndet_cov =  ncm_matrix_cholesky_lndet (cov);
-  
-  
+
+  cov       = ncm_data_gauss_cov_peek_cov (NCM_DATA_GAUSS_COV (data_mvnd));
+  lndet_cov = ncm_matrix_cholesky_lndet (cov);
+
+
   switch (test->kernel_type)
   {
     case NCM_STATS_DIST_KERNEL_GAUSS:
     {
       gdouble norm_test    = 0.5 * (test->dim * ncm_c_ln2pi () + lndet_cov);
       const gdouble lnnorm = ncm_stats_dist_kernel_get_lnnorm (test->kernel, cov);
-      
+
       ncm_assert_cmpdouble_e (norm_test, ==, lnnorm, 1.0e-14, 0.0);
       break;
     }
@@ -250,59 +250,59 @@ test_ncm_stats_dist_kernel_norm (TestNcmStatsDistKernel *test, gconstpointer pda
       const gdouble chol_lnnorm = 0.5 * lndet_cov;
       const gdouble nc_lnnorm   = (d / 2.0) * (ncm_c_lnpi () + log (test->nu));
       const gdouble lnnorm      = ncm_stats_dist_kernel_get_lnnorm (test->kernel, cov);
-      
+
       ncm_assert_cmpdouble_e (lg_lnnorm + chol_lnnorm + nc_lnnorm, ==, lnnorm, 1.0e-14, 0.0);
-      
+
       break;
     }
     default:
       g_assert_not_reached ();
       break;
   }
-  
+
   {
     gdouble *data         = g_new (gdouble, 2 * ntests);
     NcmVector *chi2_vec   = ncm_vector_new_full (data, ntests, 2, data, g_free);
     NcmVector *kernel_vec = ncm_vector_new (ntests);
-    
+
     for (i = 0; i < ntests; i++)
     {
       const gdouble chi2_i = g_test_rand_double_range (0.0, 1.0e2);
-      
+
       ncm_vector_set (chi2_vec, i, chi2_i);
     }
-    
+
     ncm_stats_dist_kernel_eval_unnorm_vec (test->kernel, chi2_vec, kernel_vec);
-    
+
     switch (test->kernel_type)
     {
       case NCM_STATS_DIST_KERNEL_GAUSS:
-      
+
         for (i = 0; i < ntests; i++)
         {
           gdouble eval_test = exp (-0.5 * ncm_vector_get (chi2_vec, i));
-          
+
           ncm_assert_cmpdouble_e (ncm_vector_get (kernel_vec, i), ==, ncm_stats_dist_kernel_eval_unnorm (test->kernel, ncm_vector_get (chi2_vec, i)), 1.0e-15, 0.0);
           ncm_assert_cmpdouble_e (eval_test, ==, ncm_stats_dist_kernel_eval_unnorm (test->kernel, ncm_vector_get (chi2_vec, i)), 1.0e-15, 0.0);
         }
-        
+
         break;
       case NCM_STATS_DIST_KERNEL_ST3:
-      
+
         for (i = 0; i < ntests; i++)
         {
           gdouble eval_test = pow (1 + ncm_vector_get (chi2_vec, i) / test->nu, -0.5 * (test->nu + test->dim));
-          
+
           ncm_assert_cmpdouble_e (ncm_vector_get (kernel_vec, i), ==, ncm_stats_dist_kernel_eval_unnorm (test->kernel, ncm_vector_get (chi2_vec, i)), 1.0e-15, 0.0);
           ncm_assert_cmpdouble_e (eval_test, ==, ncm_stats_dist_kernel_eval_unnorm (test->kernel, ncm_vector_get (chi2_vec, i)), 1.0e-15, 0.0);
         }
-        
+
         break;
       default:
         g_assert_not_reached ();
         break;
     }
-    
+
     ncm_vector_free (chi2_vec);
     ncm_vector_free (kernel_vec);
   }
@@ -334,25 +334,25 @@ test_ncm_stats_dist_kernel_sum (TestNcmStatsDistKernel *test, gconstpointer pdat
   gdouble lnt_max1       = GSL_NEGINF;
   guint i_max0           = 0;
   guint i_max1           = 0;
-  
+
   for (i = 0; i < n; i++)
   {
     const gdouble j = g_test_rand_double_range (1.0, 200.0);
     const gdouble k = g_test_rand_double_range (1.0, 200.0);
     const gdouble l = g_test_rand_double_range (1.0, 200.0);
-    
+
     ncm_vector_set (weights, i, j);
     ncm_vector_set (lnnorms_vec, i, k);
     ncm_vector_set (chi2, i, l);
   }
-  
-  
+
+
   for (i = 0; i < n; i++)
   {
     const gdouble chi2_i = ncm_vector_fast_get (chi2, i);
     const gdouble w_i    = ncm_vector_fast_get (weights, i);
     const gdouble lnu_i  = ncm_vector_fast_get (lnnorms_vec, i);
-    
+
     switch (test->kernel_type)
     {
       case NCM_STATS_DIST_KERNEL_GAUSS:
@@ -367,59 +367,59 @@ test_ncm_stats_dist_kernel_sum (TestNcmStatsDistKernel *test, gconstpointer pdat
         g_assert_not_reached ();
         break;
     }
-    
+
     if (lnt_i0 > lnt_max0)
     {
       lnt_max0 = lnt_i0;
       i_max0   = i;
     }
-    
+
     if (lnt_i1 > lnt_max1)
     {
       lnt_max1 = lnt_i1;
       i_max1   = i;
     }
-    
+
     g_array_insert_val (t_array0, i, lnt_i0);
     g_array_insert_val (t_array1, i, lnt_i1);
   }
-  
+
   ncm_stats_dist_kernel_eval_sum0_gamma_lambda (test->kernel, chi2, weights, lnnorms_vec, &gamma0, &lambda0);
   ncm_stats_dist_kernel_eval_sum1_gamma_lambda (test->kernel, chi2, weights, lnnorm, &gamma1, &lambda1);
-  
+
   for (i = 0; i < i_max0; i++)
   {
     lambda_test0 += exp (g_array_index (t_array0, gdouble, i) - lnt_max0);
     ncm_assert_cmpdouble_e (g_array_index (t_array0, gdouble, i), <, gamma0, 1.0e-15, 0.0);
   }
-  
+
   for (i = 0; i < i_max1; i++)
   {
     lambda_test1 += exp (g_array_index (t_array1, gdouble, i) - lnt_max1);
     ncm_assert_cmpdouble_e (g_array_index (t_array1, gdouble, i) - lnnorm, <, gamma1, 1.0e-15, 0.0);
   }
-  
+
   for (i = i_max0 + 1; i < n; i++)
   {
     lambda_test0 += exp (g_array_index (t_array0, gdouble, i) - lnt_max0);
     ncm_assert_cmpdouble_e (g_array_index (t_array0, gdouble, i), <, gamma0, 1.0e-15, 0.0);
   }
-  
+
   for (i = i_max1 + 1; i < n; i++)
   {
     lambda_test1 += exp (g_array_index (t_array1, gdouble, i) - lnt_max1);
     ncm_assert_cmpdouble_e (g_array_index (t_array1, gdouble, i) - lnnorm, <, gamma1, 1.0e-15, 0.0);
   }
-  
+
   ncm_assert_cmpdouble_e (lnt_max0, ==, gamma0, 1.0e-15, 0.0);
   ncm_assert_cmpdouble_e (lnt_max1 - lnnorm, ==, gamma1, 1.0e-15, 0.0);
-  
+
   ncm_assert_cmpdouble_e (lambda_test0, ==, lambda0, 1.0e-15, 0.0);
   ncm_assert_cmpdouble_e (lambda_test1, ==, lambda1, 1.0e-15, 0.0);
-  
+
   ncm_assert_cmpdouble_e (lambda_test0 + exp (lnt_max0), ==, lambda0 + exp (gamma0), 1.0e-15, 0.0);
   ncm_assert_cmpdouble_e (lambda_test1 + exp (lnt_max1 - lnnorm), ==, lambda1 + exp (gamma1), 1.0e-15, 0.0);
-  
+
   ncm_vector_free (weights);
   ncm_vector_free (chi2);
   ncm_vector_free (lnnorms_vec);
@@ -438,42 +438,42 @@ test_ncm_stats_dist_kernel_sample (TestNcmStatsDistKernel *test, gconstpointer p
   NcmVector *mu           = ncm_vector_new (test->dim);
   NcmStatsVec *test_stats = ncm_stats_vec_new (test->dim, NCM_STATS_VEC_VAR, FALSE);
   const guint ntests      = 300 * g_test_rand_int_range (1, 5);
-  
+
   for (i = 0; i < test->dim; i++)
   {
     const gdouble val1 = g_test_rand_double_range (1.0, 200.0);
-    
+
     ncm_vector_set (mu, i, val1);
-    
+
     for (j = 0; j < test->dim; j++)
     {
       const gdouble val2 = g_test_rand_double_range (1.0, 200.0);
-      
+
       ncm_matrix_set (cov_decomp, i, j, val2);
     }
   }
-  
+
   ncm_matrix_cholesky_decomp (cov_decomp, 'U');
-  
+
   while (test->nfail < 20)
   {
     for (i = 0; i < ntests; i++)
     {
       NcmVector *x_test = ncm_vector_new (test->dim);
-      
+
       ncm_stats_dist_kernel_sample (test->kernel, cov_decomp, href, mu, x_test, rng);
-      
+
       ncm_stats_vec_append (test_stats, x_test, FALSE);
     }
-    
+
     for (i = 0; i < test->dim; i++)
     {
       gdouble dif_i;
-      
+
       dif_i = abs ((ncm_stats_vec_get_mean (test_stats, i) - ncm_vector_get (mu, i)) / ncm_vector_get (mu, i));
       dif   = dif + dif_i;
     }
-    
+
     if (dif >= (test->dim * 0.2))
     {
       dif         = 0.0;
@@ -484,11 +484,11 @@ test_ncm_stats_dist_kernel_sample (TestNcmStatsDistKernel *test, gconstpointer p
       test->nfail = 20;
     }
   }
-  
+
   g_assert_cmpfloat (dif, <, (test->dim * 0.2));
-  
-  
-  
+
+
+
   ncm_vector_free (mu);
   ncm_matrix_free (cov_decomp);
   ncm_rng_free (rng);
